from risc_generator import FPQ1Contract, Insuree
from risc_generator.content import fr, en
from tests.domain.contract.contract_base import ContractTestBase

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractDeclarationTest(ContractTestBase):
    def test_givenAFRContractNoRebates_whenGenerateItem7Text_thenReturnProperFRText(self):
        a_association_rebate = False
        payment_method = "bill"

        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=self.a_suspension,
            number_of_claims_past_years=self.a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_7_text(
            association_rebate=fpq1_contract.insuree.association_rebate, payment_method=fpq1_contract.payment_method
        )

        expected = (
            "Article 7. INFORMATIONS POUR L'ASSURÉ DÉSIGNÉ\n"
            "\t Consentement : Si une autorisation nous a été donnée afin d'obtenir des informations de crédit "
            "pour l'émission du présent contrat, veuillez noter que les renseignements obtenus seront mis à "
            "jour et utilisés pour votre renouvellement et pour le traitement de vos dossiers d'assurances "
            "générales. Ce consentement peut être révoqué en tout temps en nous avisant par écrit.\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithAssociationRebate_whenGenerateItem7Text_thenReturnProperFRText(self):
        a_association_rebate = True
        payment_method = "bill"

        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=self.a_suspension,
            number_of_claims_past_years=self.a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_7_text(
            association_rebate=fpq1_contract.insuree.association_rebate, payment_method=fpq1_contract.payment_method
        )

        expected = (
            "Article 7. INFORMATIONS POUR L'ASSURÉ DÉSIGNÉ\n"
            "\t La prime tient comptes des réductions suivantes :\n"
            "- En tant que membre/client d'une association, vous avez droit au rabais suivant: 10%.\n"
            "\t Consentement : Si une autorisation nous a été donnée afin d'obtenir des informations de crédit "
            "pour l'émission du présent contrat, veuillez noter que les renseignements obtenus seront mis à "
            "jour et utilisés pour votre renouvellement et pour le traitement de vos dossiers d'assurances "
            "générales. Ce consentement peut être révoqué en tout temps en nous avisant par écrit.\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithPaymentRebate_whenGenerateItem7Text_thenReturnProperFRText(self):
        a_association_rebate = False
        payment_method = "pre-authorize"

        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=self.a_suspension,
            number_of_claims_past_years=self.a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_7_text(
            association_rebate=fpq1_contract.insuree.association_rebate, payment_method=fpq1_contract.payment_method
        )

        expected = (
            "Article 7. INFORMATIONS POUR L'ASSURÉ DÉSIGNÉ\n"
            "\t La prime tient comptes des réductions suivantes :\n"
            "- 2 % pour avoir choisi le Prélèvement préautorisé comme mode de paiement de votre prime\n"
            "\t Consentement : Si une autorisation nous a été donnée afin d'obtenir des informations de crédit "
            "pour l'émission du présent contrat, veuillez noter que les renseignements obtenus seront mis à "
            "jour et utilisés pour votre renouvellement et pour le traitement de vos dossiers d'assurances "
            "générales. Ce consentement peut être révoqué en tout temps en nous avisant par écrit.\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithAssociationAndPaymentRebate_whenGenerateItem7Text_thenReturnProperFRText(self):
        a_association_rebate = True
        payment_method = "pre-authorize"

        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=self.a_suspension,
            number_of_claims_past_years=self.a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_7_text(
            association_rebate=fpq1_contract.insuree.association_rebate, payment_method=fpq1_contract.payment_method
        )

        expected = (
            "Article 7. INFORMATIONS POUR L'ASSURÉ DÉSIGNÉ\n"
            "\t La prime tient comptes des réductions suivantes :\n"
            "- En tant que membre/client d'une association, vous avez droit au rabais suivant: 10%.\n"
            "- 2 % pour avoir choisi le Prélèvement préautorisé comme mode de paiement de votre prime\n"
            "\t Consentement : Si une autorisation nous a été donnée afin d'obtenir des informations de crédit "
            "pour l'émission du présent contrat, veuillez noter que les renseignements obtenus seront mis à "
            "jour et utilisés pour votre renouvellement et pour le traitement de vos dossiers d'assurances "
            "générales. Ce consentement peut être révoqué en tout temps en nous avisant par écrit.\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractNoRebates_whenGenerateItem7Text_thenReturnProperENText(self):
        a_association_rebate = False
        payment_method = "bill"

        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=self.a_suspension,
            number_of_claims_past_years=self.a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_7_text(
            association_rebate=fpq1_contract.insuree.association_rebate, payment_method=fpq1_contract.payment_method
        )

        expected = (
            "Item 7. INFORMATION FOR THE NAMED INSURED\n"
            "\t Consent:If authorization has been granted for us to obtain credit information for the purposes of "
            "issuing this contract, please note that the information obtained will be updated and used for renewing "
            "your contract and for processing your property and casualty insurance files. This consent can be revoked "
            "at any time by notifying us in writing.\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithAssociationRebate_whenGenerateItem7Text_thenReturnProperENText(self):
        a_association_rebate = True
        payment_method = "bill"

        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=self.a_suspension,
            number_of_claims_past_years=self.a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_7_text(
            association_rebate=fpq1_contract.insuree.association_rebate, payment_method=fpq1_contract.payment_method
        )

        expected = (
            "Item 7. INFORMATION FOR THE NAMED INSURED\n"
            "\t The premium takes into account the following discounts:\n"
            "- As a member/customer of an association, you are eligible for the following discount: 10%.\n"
            "\t Consent:If authorization has been granted for us to obtain credit information for the purposes of "
            "issuing this contract, please note that the information obtained will be updated and used for renewing "
            "your contract and for processing your property and casualty insurance files. This consent can be revoked "
            "at any time by notifying us in writing.\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithPaymentRebate_whenGenerateItem7Text_thenReturnProperENText(self):
        a_association_rebate = False
        payment_method = "pre-authorize"

        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=self.a_suspension,
            number_of_claims_past_years=self.a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_7_text(
            association_rebate=fpq1_contract.insuree.association_rebate, payment_method=fpq1_contract.payment_method
        )

        expected = (
            "Item 7. INFORMATION FOR THE NAMED INSURED\n"
            "\t The premium takes into account the following discounts:\n"
            "- 2% for choosing Pre-authorized payment as your premium payment method\n"
            "\t Consent:If authorization has been granted for us to obtain credit information for the purposes of "
            "issuing this contract, please note that the information obtained will be updated and used for renewing "
            "your contract and for processing your property and casualty insurance files. This consent can be revoked "
            "at any time by notifying us in writing.\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithAssociationAndPaymentRebate_whenGenerateItem7Text_thenReturnProperENText(self):
        a_association_rebate = True
        payment_method = "pre-authorize"

        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=self.a_suspension,
            number_of_claims_past_years=self.a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_7_text(
            association_rebate=fpq1_contract.insuree.association_rebate, payment_method=fpq1_contract.payment_method
        )

        expected = (
            "Item 7. INFORMATION FOR THE NAMED INSURED\n"
            "\t The premium takes into account the following discounts:\n"
            "- As a member/customer of an association, you are eligible for the following discount: 10%.\n"
            "- 2% for choosing Pre-authorized payment as your premium payment method\n"
            "\t Consent:If authorization has been granted for us to obtain credit information for the purposes of "
            "issuing this contract, please note that the information obtained will be updated and used for renewing "
            "your contract and for processing your property and casualty insurance files. This consent can be revoked "
            "at any time by notifying us in writing.\n"
        )

        self.assertEqual(expected, actual)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq2_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 2\n"
        "Vehicles of which named insured is not owner and when driven by named drivers\n"
        "(Section A)\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Additional insurance premium payable:\n"
        "\t Amount payable:  \n"
        "\t Due date:  \n"
        "Endorsement description\n"
        "This endorsement extends coverage under Section A of the insurance contract by adding the following "
        "paragraph to Article 2 entitled ''Insured vehicles'':\n"
        "''a vehicle of the   type when driven, at the time of the loss, by one of the "
        "following persons:\n"
        "NAME | AGE | RELATIONSHIP TO NAMED INSURED\n"
        "1. | | \n"
        "2. | | \n"
        "3. | | \n"
        "4. | | \n"
        "For the vehicle to be considered an ''insured vehicle'' under Section A, the following conditions "
        "must be met:\n"
        "1. At the time of the loss, the vehicle is not being driven in connection with a garage business.\n"
        "2. The owner or frequent user of the vehicle is not one of the following persons:\n"
        "\t the named insured or anyone whose domicile is the same as that of the named insured;\n"
        "\t a person mentioned in the table above or anyone whose domicile is the same as that of "
        "such person.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "3. The vehicle is not provided by an employer of:n"
        "\t the named insured or anyone whose domicile is the same as that of the named insured;\n"
        "\t a person mentioned in the table above or anyone whose domicile is the same as that of "
        "such person.\n"
        "4. The vehicle is not appropriated to a use:\n"
        "\t as a taxicab, bus or coach; or\n"
        "\t for commercial delivery.''\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



back_of_certificates_text = (
    "CERTIFICAT D'ASSURANCE RESPONSABILITÉ AUTOMOBILE\n"
    "Le présent certiﬁcat est assujetti aux dispositions et conditions de la police "
    "d'assurance automobile de l'Assureur.\n"
    "Ce certiﬁcat atteste que la personne susnommée est assurée contre la responsabilité "
    "pour blessures et dommages aux biens découlant de l'usage du véhicule ci-décrit, "
    "conformément aux limites minimales exigées par les lois d'assurance en vigueur dans "
    "chacune des provinces canadiennes.\n"
    "AVERTISSEMENT - Quiconque émet ou présente un tel certiﬁcat comme preuve d'une "
    "police d'assurance responsabilité qui effectivement n'est pas en vigueur, est coupable "
    "d'une infraction passible d'une forte amende et/ou d'emprisonnement et suspension "
    "de son permis.\n"
    "Tout sinistre devrait être rapporté immédiatement.\n\n"
    "CANADA INTER-PROVINCE MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
    "This certiﬁcate is subject to the terms and conditions of the Insurers standard "
    "automobile policy.\n"
    "This certiﬁes that the party named herein is insured against liability for bodily injury and "
    "property damage by reason of the operation of the motor vehicle described herein, for an "
    "amount not less than the statutory minimum requirements of every province of Canada.\n"
    "WARNING - Any person who issues or presents such a certiﬁcate as proof of insurance "
    "when no policy exists is guilty of fraud and liable to a heavy ﬁ ne and/or imprisonment. "
    "His driving permit may also be suspended.\n"
    "All loss should be reported immediately.\n\n"
)



import datetime
from unittest import TestCase

from risc_generator import Protections, Insuree
from risc_generator.faker.contract_faker.premium_faker import QuebecPremiumFaker

INCLUDE = "include"
EXCLUDE = "exclude"


class PremiumFakerTests(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.premium_faker = QuebecPremiumFaker()

    def assertNoPremiumOnProtection(self, actual, protection):
        self.assertIsNone(actual.get(protection))

    def assertPremiumOnProtection(self, actual, protection):
        self.assertGreater(actual.get(protection), 0)

    def test_whenPremiumFaker_withB2B3_thenReturnCorrectPremium(
        self,
    ):
        a_sex = "male"
        a_name = "A Name"
        a_date_of_birth = datetime.date(year=2020, month=1, day=1)
        an_address = "An address\nA city, QC, A1A 1A1"
        a_suspension = False
        a_number_of_claims_past_years = 1
        a_association_rebate = False
        a_insuree = Insuree(
            sex=a_sex,
            name=a_name,
            birth_date=a_date_of_birth,
            address=an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
        }

        a_protections = Protections(protections=a_dict_of_protections)

        actual_premium = self.premium_faker.premium(protections=a_protections, insuree=a_insuree)

        self.assertPremiumOnProtection(actual_premium, "LiabilityCoveragePremium")

        self.assertPremiumOnProtection(actual_premium, "PropertyDamageB2CoveragePremium")
        self.assertPremiumOnProtection(actual_premium, "PropertyDamageB3CoveragePremium")

        # No premium for B1 and B4
        self.assertNoPremiumOnProtection(actual_premium, "PropertyDamageB1CoveragePremium")
        self.assertNoPremiumOnProtection(actual_premium, "PropertyDamageB4CoveragePremium")

    def test_whenPremiumFaker_withB1_thenReturnCorrectPremium(
        self,
    ):
        a_sex = "male"
        a_name = "A Name"
        a_date_of_birth = datetime.date(year=2020, month=1, day=1)
        an_address = "An address\nA city, QC, A1A 1A1"
        a_suspension = False
        a_number_of_claims_past_years = 1
        a_association_rebate = False
        a_insuree = Insuree(
            sex=a_sex,
            name=a_name,
            birth_date=a_date_of_birth,
            address=an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": 500,
            "PropertyDamageB2Coverage": INCLUDE,
            "PropertyDamageB3Coverage": INCLUDE,
            "PropertyDamageB4Coverage": EXCLUDE,
        }

        a_protections = Protections(protections=a_dict_of_protections)

        actual_premium = self.premium_faker.premium(protections=a_protections, insuree=a_insuree)

        self.assertPremiumOnProtection(actual_premium, "LiabilityCoveragePremium")

        self.assertPremiumOnProtection(actual_premium, "PropertyDamageB1CoveragePremium")

        # No premium for B2, B3 and B4 since it is covered by B1
        self.assertNoPremiumOnProtection(actual_premium, "PropertyDamageB2CoveragePremium")
        self.assertNoPremiumOnProtection(actual_premium, "PropertyDamageB3CoveragePremium")
        self.assertNoPremiumOnProtection(actual_premium, "PropertyDamageB4CoveragePremium")

    def test_whenPremiumFaker_withB4_thenReturnCorrectPremium(
        self,
    ):
        a_sex = "male"
        a_name = "A Name"
        a_date_of_birth = datetime.date(year=2020, month=1, day=1)
        an_address = "An address\nA city, QC, A1A 1A1"
        a_suspension = False
        a_number_of_claims_past_years = 1
        a_association_rebate = False
        a_insuree = Insuree(
            sex=a_sex,
            name=a_name,
            birth_date=a_date_of_birth,
            address=an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": EXCLUDE,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": 250,
            "FAQ27ACoverage": INCLUDE,
        }

        a_protections = Protections(protections=a_dict_of_protections)

        actual_premium = self.premium_faker.premium(protections=a_protections, insuree=a_insuree)

        self.assertPremiumOnProtection(actual_premium, "LiabilityCoveragePremium")

        self.assertPremiumOnProtection(actual_premium, "PropertyDamageB4CoveragePremium")

        # No premium for B2, B3 and B4 since it is covered by B1
        self.assertNoPremiumOnProtection(actual_premium, "PropertyDamageB2CoveragePremium")
        self.assertNoPremiumOnProtection(actual_premium, "PropertyDamageB3CoveragePremium")
        self.assertNoPremiumOnProtection(actual_premium, "PropertyDamageB1CoveragePremium")

    def test_whenPremiumFaker_withEndorsementsWithPremium_thenReturnCorrectPremium(
        self,
    ):
        a_sex = "male"
        a_name = "A Name"
        a_date_of_birth = datetime.date(year=2020, month=1, day=1)
        an_address = "An address\nA city, QC, A1A 1A1"
        a_suspension = False
        a_number_of_claims_past_years = 1
        a_association_rebate = False
        a_insuree = Insuree(
            sex=a_sex,
            name=a_name,
            birth_date=a_date_of_birth,
            address=an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=a_association_rebate,
        )

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": EXCLUDE,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ20ACoverage": INCLUDE,
            "FAQ33Coverage": INCLUDE,
            "FAQ34Coverage": INCLUDE,
            "FAQ43Coverage": INCLUDE,
        }

        a_protections = Protections(protections=a_dict_of_protections)

        actual_premium = self.premium_faker.premium(protections=a_protections, insuree=a_insuree)

        self.assertPremiumOnProtection(actual_premium, "LiabilityCoveragePremium")

        self.assertPremiumOnProtection(actual_premium, "FAQ20ACoveragePremium")
        self.assertPremiumOnProtection(actual_premium, "FAQ33CoveragePremium")
        self.assertPremiumOnProtection(actual_premium, "FAQ34CoveragePremium")
        self.assertPremiumOnProtection(actual_premium, "FAQ43CoveragePremium")



import re
from datetime import datetime, timedelta
from typing import Union

from babel.dates import format_date
from dateutil.relativedelta import relativedelta
from faker import Faker

from .premium import Premium
from ..content import fr, en
from ..domain.insuree import Insuree
from ..domain.protections import Protections
from ..domain.vehicle import Vehicle


class FPQ1Contract:
    def __init__(
        self,
        client_number: str,
        contract_start_date: datetime.date,
        insuree: Insuree,
        vehicle: Vehicle,
        payment_method: str,
        protections: Protections,
        premium: Premium,
        financing: Union[None, str],
        locale: str,
    ):
        """
        A FPQ1 contract.

        client_number (str): A unique client number.
        start_date (datetime.date): Date the contract start (the coverage).
        header_contract_date (datetime.date): Date the contract was printed.
        insuree (Insuree): Insuree of the contract.
        vehicle (Vehicle): The vehicle details.
        payment_method (str): The type of payment method. Either 'pre-authorize' or 'bill'.
        protections (Protections): The protections.
        premium (Premium): A premium base on the protections and the insuree of the contract.
        locale (str): The locale language setting to use for simulation. Can either be `'fr_CA'` or `'en_CA'`.
        """
        super().__init__()
        self.locale = locale
        if self.locale == "fr_CA":
            self.ownership_rental = "Location"
            self.ownership_purchase = "Achat"
        else:
            self.ownership_rental = "Rental"
            self.ownership_purchase = "Purchase"

        self.client_number = client_number
        self.contract_start_date = contract_start_date

        # Date the paperwork is printed:
        # A contract cannot be legally dated more than 90 days before the start of the contract.
        # Thus, the paperwork with the contract cannot be date more than 90 days before the contract start date.
        # To do so, we cannot use self.date_faker.date_between(start_date=-90d) since it will create a start date
        # 90 days before today (the time the Python script is run) rather than 90 days before the contract start date.
        # The following code was taken from the date_between date_parsing function.
        # Namely, this code does a timedelta of minus 90 days init with a dict plus the contract_start_date.
        # Thus, it will randomly select a date of at most 90 days before the start of the contract.
        contract_paperwork_date = contract_start_date + timedelta(**{"days": -90})
        date_faker = Faker(locale=self.locale)
        header_contract_date = date_faker.date_between(start_date=contract_paperwork_date, end_date=contract_start_date)

        self.header_contract_date = header_contract_date
        self.contract_end_date = contract_start_date + relativedelta(years=1)
        self.insuree = insuree
        self.vehicle = vehicle

        self.payment_method = payment_method

        self.protections = protections

        self.premium = premium

        self.financing = financing

        self.policy = "001"  # We set it to 1 since we assume (for now) only new clients
        self.policy_number = f"{self.client_number}-{self.policy}"

        self._dynamic_import_text_generation_method()

        self._dynamic_import_method_endorsement_text_generation()

    def formatted_text_date(self) -> str:
        """
        Method to format the date in a text format in the locale language

        Return:
            The date in a text format.
        """
        prefix = ""
        if "fr" in self.locale:
            prefix = "Le "
        return prefix + format_date(self.header_contract_date, format='long', locale=self.locale)

    def car_ownership(self) -> str:
        """
        Method to get the car ownership type (rental or purchase) in the locale language.

        Return:
            The car ownership.
        """
        if self.protections.is_protected("FAQ5ACoverage"):
            # The car is a rental one since this endorsement is use for rented car
            car_ownership = self.ownership_rental
        else:
            # The car was purchase (with or without a creditor)
            car_ownership = self.ownership_purchase
        return car_ownership

    def car_with_financing(self) -> Union[None, str]:
        """
        Method to evaluate if the car is financed with a creditor (e.g. protections include a FAQ23a coverage)
        or a lessor (e.g. protections include a FAQ5a coverage).

        Return:
            A none if no financing or a string of the financing type (e.g. `'creditor'` or `'lessor'`).
        """
        if self.protections.is_protected("FAQ23ACoverage"):
            return_value = 'creditor'
        elif self.protections.is_protected("FAQ5ACoverage"):
            return_value = 'lessor'
        else:
            return_value = None
        return return_value

    def __str__(self):
        return f"FPQ1 contract with the following protections:\n{str(self.protections)}"

    __repr__ = __str__

    def _dynamic_import_text_generation_method(self) -> None:
        # pylint: disable=too-many-branches
        """
        Method to dynamically handle the selection of the proper function to generate text in a specific language.
        """

        # Add header_template method
        if "fr" in self.locale:
            header_import = fr.header.generate_header_text
        else:
            header_import = en.header.generate_header_text
        setattr(FPQ1Contract, "generate_header_text", header_import)

        # Add important instructions method
        if "fr" in self.locale:
            important_instructions_import = fr.important_instructions.generate_important_instructions_text
        else:
            important_instructions_import = en.important_instructions.generate_important_instructions_text
        setattr(FPQ1Contract, "generate_important_instructions_text", important_instructions_import)

        # Add declaration method
        if "fr" in self.locale:
            declaration_import = fr.declaration.generate_declaration_text
        else:
            declaration_import = en.declaration.generate_declaration_text
        setattr(FPQ1Contract, "generate_declaration_text", declaration_import)

        # Add cancellation of policy method
        if "fr" in self.locale:
            cancellation_of_policy_import = fr.cancellation_of_policy.generate_cancellation_of_policy_text
        else:
            cancellation_of_policy_import = en.cancellation_of_policy.generate_cancellation_of_policy_text
        setattr(FPQ1Contract, "generate_cancellation_of_policy_text", cancellation_of_policy_import)

        # Add certificates method
        if "fr" in self.locale:
            certificates_import = fr.certificates.generate_certificates_text
        else:
            certificates_import = en.certificates.generate_certificates_text
        setattr(FPQ1Contract, "generate_certificates_text", certificates_import)

        # Add bill method
        if "fr" in self.locale:
            bill_import = fr.bill.generate_bill_text
        else:
            bill_import = en.bill.generate_bill_text
        setattr(FPQ1Contract, "generate_bill_text", bill_import)

        # Add client advantages method
        if "fr" in self.locale:
            client_advantages_import = fr.client_advantages.generate_client_advantages_text
        else:
            client_advantages_import = en.client_advantages.generate_client_advantages_text
        setattr(FPQ1Contract, "generate_client_advantages_text", client_advantages_import)

        # Add FPQ1 method
        if "fr" in self.locale:
            fpq1_method = fr.fpq_1.generate_fpq1_text
        else:
            fpq1_method = en.fpq_1.generate_fpq1_text
        setattr(FPQ1Contract, "generate_fpq1_text", fpq1_method)

    def _dynamic_import_method_endorsement_text_generation(self) -> None:
        # pylint: disable=too-many-locals, too-many-statements
        """
        Dynamically import all the method for the endorsement text generation
        """
        if "fr" in self.locale:
            faq_2_import = fr.endorsements.generate_faq2_endorsement_text
            faq_3_import = fr.endorsements.generate_faq3_endorsement_text
            faq_5a_import = fr.endorsements.generate_faq5a_endorsement_text
            faq_8_import = fr.endorsements.generate_faq8_endorsement_text
            faq_9_import = fr.endorsements.generate_faq9_endorsement_text
            faq_13_import = fr.endorsements.generate_faq13_endorsement_text
            faq_16_import = fr.endorsements.generate_faq16_endorsement_text
            faq_19_import = fr.endorsements.generate_faq19_endorsement_text
            faq_20_import = fr.endorsements.generate_faq20_endorsement_text
            faq_20a_import = fr.endorsements.generate_faq20a_endorsement_text
            faq_23a_import = fr.endorsements.generate_faq23a_endorsement_text
            faq_25_import = fr.endorsements.generate_faq25_endorsement_text
            faq_27_import = fr.endorsements.generate_faq27_endorsement_text
            faq_27a_import = fr.endorsements.generate_faq27a_endorsement_text
            faq_28_import = fr.endorsements.generate_faq28_endorsement_text
            faq_30_import = fr.endorsements.generate_faq30_endorsement_text
            faq_31_import = fr.endorsements.generate_faq31_endorsement_text
            faq_32_import = fr.endorsements.generate_faq32_endorsement_text
            faq_33_import = fr.endorsements.generate_faq33_endorsement_text
            faq_34_import = fr.endorsements.generate_faq34_endorsement_text
            faq_34ab_import = fr.endorsements.generate_faq34ab_endorsement_text
            faq_37ab_import = fr.endorsements.generate_faq37ab_endorsement_text
            faq_40_import = fr.endorsements.generate_faq40_endorsement_text
            faq_41_import = fr.endorsements.generate_faq41_endorsement_text
            faq_43_import = fr.endorsements.generate_faq43_endorsement_text
        else:
            faq_2_import = en.endorsements.generate_faq2_endorsement_text
            faq_3_import = en.endorsements.generate_faq3_endorsement_text
            faq_5a_import = en.endorsements.generate_faq5a_endorsement_text
            faq_8_import = en.endorsements.generate_faq8_endorsement_text
            faq_9_import = en.endorsements.generate_faq9_endorsement_text
            faq_13_import = en.endorsements.generate_faq13_endorsement_text
            faq_16_import = en.endorsements.generate_faq16_endorsement_text
            faq_19_import = en.endorsements.generate_faq19_endorsement_text
            faq_20_import = en.endorsements.generate_faq20_endorsement_text
            faq_20a_import = en.endorsements.generate_faq20a_endorsement_text
            faq_23a_import = en.endorsements.generate_faq23a_endorsement_text
            faq_25_import = en.endorsements.generate_faq25_endorsement_text
            faq_27_import = en.endorsements.generate_faq27_endorsement_text
            faq_27a_import = en.endorsements.generate_faq27a_endorsement_text
            faq_28_import = en.endorsements.generate_faq28_endorsement_text
            faq_30_import = en.endorsements.generate_faq30_endorsement_text
            faq_31_import = en.endorsements.generate_faq31_endorsement_text
            faq_32_import = en.endorsements.generate_faq32_endorsement_text
            faq_33_import = en.endorsements.generate_faq33_endorsement_text
            faq_34_import = en.endorsements.generate_faq34_endorsement_text
            faq_34ab_import = en.endorsements.generate_faq34ab_endorsement_text
            faq_37ab_import = en.endorsements.generate_faq37ab_endorsement_text
            faq_40_import = en.endorsements.generate_faq40_endorsement_text
            faq_41_import = en.endorsements.generate_faq41_endorsement_text
            faq_43_import = en.endorsements.generate_faq43_endorsement_text

        setattr(FPQ1Contract, "generate_faq2_endorsement_text", faq_2_import)
        setattr(FPQ1Contract, "generate_faq3_endorsement_text", faq_3_import)
        setattr(FPQ1Contract, "generate_faq5a_endorsement_text", faq_5a_import)
        setattr(FPQ1Contract, "generate_faq8_endorsement_text", faq_8_import)
        setattr(FPQ1Contract, "generate_faq9_endorsement_text", faq_9_import)
        setattr(FPQ1Contract, "generate_faq13_endorsement_text", faq_13_import)
        setattr(FPQ1Contract, "generate_faq16_endorsement_text", faq_16_import)
        setattr(FPQ1Contract, "generate_faq19_endorsement_text", faq_19_import)
        setattr(FPQ1Contract, "generate_faq20_endorsement_text", faq_20_import)
        setattr(FPQ1Contract, "generate_faq20a_endorsement_text", faq_20a_import)
        setattr(FPQ1Contract, "generate_faq23a_endorsement_text", faq_23a_import)
        setattr(FPQ1Contract, "generate_faq25_endorsement_text", faq_25_import)
        setattr(FPQ1Contract, "generate_faq27_endorsement_text", faq_27_import)
        setattr(FPQ1Contract, "generate_faq27a_endorsement_text", faq_27a_import)
        setattr(FPQ1Contract, "generate_faq28_endorsement_text", faq_28_import)
        setattr(FPQ1Contract, "generate_faq30_endorsement_text", faq_30_import)
        setattr(FPQ1Contract, "generate_faq31_endorsement_text", faq_31_import)
        setattr(FPQ1Contract, "generate_faq32_endorsement_text", faq_32_import)
        setattr(FPQ1Contract, "generate_faq33_endorsement_text", faq_33_import)
        setattr(FPQ1Contract, "generate_faq34_endorsement_text", faq_34_import)
        setattr(FPQ1Contract, "generate_faq34ab_endorsement_text", faq_34ab_import)
        setattr(FPQ1Contract, "generate_faq37a_endorsement_text", faq_37ab_import)  # Same as 37 AB
        setattr(FPQ1Contract, "generate_faq37ab_endorsement_text", faq_37ab_import)
        setattr(FPQ1Contract, "generate_faq40_endorsement_text", faq_40_import)
        setattr(FPQ1Contract, "generate_faq41_endorsement_text", faq_41_import)
        setattr(FPQ1Contract, "generate_faq43_endorsement_text", faq_43_import)

    def generate_contract_text(self) -> str:
        """
        Generate contract text.

        Return:
            The contract text.
        """

        header_text = self.generate_header_text()

        important_instructions = self.generate_important_instructions_text()

        declarations = self.generate_declaration_text()

        cancellation_of_policy = self.generate_cancellation_of_policy_text()

        bill = self.generate_bill_text()

        certificates = self.generate_certificates_text()

        client_advantages = self.generate_client_advantages_text()

        fpq1_amf_text = self.generate_fpq1_text()

        endorsements = self.generate_endorsements_text()

        contract_text = (
            header_text
            + important_instructions
            + declarations
            + cancellation_of_policy
            + bill
            + certificates
            + client_advantages
            + fpq1_amf_text
            + endorsements
        )

        return contract_text

    def generate_endorsements_text(self) -> str:
        """
        Method to generate the endorsements of the protections.

        Return:
            The endorsement text.
        """
        text = ""
        for endorsement in self.protections.endorsement_protections():
            endorsement_name = re.findall(r'FAQ[0-9]+[AB]*', endorsement)[0].lower()
            endorsement_text = getattr(self, f"generate_{endorsement_name}_endorsement_text")()
            text += endorsement_text + "\n"

        return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq27a_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 27a\n"
        "Civil liability resulting from damage caused to vehicles of which named insured is not owner\n"
        "(excluding vehicles provided by an employer)\n"
        "(Section A)\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Termination date: This endorsement will apply until   at 12:01 A.M. "
        "standard time at the address of the named insured or, if no date is specified, until "
        "the expiry date of the insurance contract.\n"
        "Additional insurance premium payable:\n"
        "\t Amount payable:  \n"
        "\t Due date:  \n"
        "Endorsement description\n"
        "This endorsement extends coverage under Section A of the insurance contract to the financial "
        "consequences that an insured person may incur when civilly liable for:\n"
        "\t damage caused to a vehicle of the  type or its equipment and accessories; and\n"
        "\t disappearance of the vehicle or its equipment and accessories.\n"
        "Civil liability may be contractual or extracontractual.\n"
        "Insured persons\n"
        "In this endorsement, ''insured person'' refers to:\n"
        "\t the named insured;\n"
        "\t his or her spouse;\n"
        "\t any person designated in a Q.E.F. N 2 entitled ''Vehicles of which named insured "
        "is not owner and when driven by named drivers'', attached to the insurance contract;\n"
        "\t the legal representatives and the succession of all the above listed insured persons.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "However, if the named insured is a legal person, partnership or association, "
        "''insured person'' refers to:\n"
        "\t any employee, shareholder, partner or member authorized by the named insured;\n"
        "\t their spouse;\n"
        "\t any person designated in a Q.E.F. N 2, attached to the insurance contract;\n"
        "\t the legal representatives and the succession of all the above listed insured persons.\n"
        "or solely:\n"
        "\t the following persons:  ;\n"
        "\t their spouse;\n"
        "\t any person designated in a Q.E.F. N 2, attached to the insurance contract;\n"
        "\t the legal representatives and the succession of all the above listed insured persons.\n"
        "Application\n"
        "1. The insured person must have the care, custody or control of the vehicle or its "
        "equipment and accessories.\n"
        "2. The insured person or anyone whose domicile is the same as that of the named insured "
        "must not be the owner of the vehicle or its lessee for at least one year or under a contract "
        "of leasing.\n"
        "3. The employer of an insured person or the employer of anyone whose domicile is the same as "
        "that of the named insured must not be the owner of the vehicle or its lessee for at least "
        "one year or under a contract of leasing.\n"
        "Covered perils and insurance premium\n"
        "Coverage is provided only for those perils for which a deductible or an insurance "
        "premium is shown in the table below, or entered specifically for this endorsement "
        "in the ''Declarations'' section of the insurance contract:\n"
        "SECTION A: Coverage for civil liability arising from property damage "
        "and bodily injury caused to another person\n"
        " PERILS | DEDUCTIBLE | INSURANCE PREMIUM\n"
        "Protection 1: ''All perils'', Protection 2: Perils of collision and upset, "
        "Protection 3: All perils other than collision or upset, "
        "Protection 4: Specific perils | Deductible per loss:: $ $ $ $ | $ $ $ $ \n"
        "  | Total : |  $"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Clarifications\n"
        "1. Protections 1, 2, 3 and 4 have the same meanings as in Section B of the "
        "insurance contract. The exclusions specified in that section will apply, as the case may be.\n"
        "2. An amount of insurance of  $ will apply per loss, plus expenses, costs and "
        "interest arising from a lawsuit.\n"
        "3. Where applicable, the additional coverage provided under Section A of "
        "the insurance contract may apply.\n"
        "4. The insurer agrees not to exercise any recourse against a person who, with "
        "the insured person's consent, has the care, custody or control of the vehicle or its "
        "equipment and accessories, unless that person:\n"
        "\t was engaged in a garage business at the time of the loss; or\n"
        "\t failed to comply with the insurance contract.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from unittest import TestCase

from risc_generator import QuebecPurchaseConditionFaker


class QuebecPurchaseConditionFakerTests(TestCase):
    n = 250000
    delta = 0.05  # A delta of 0.05, meaning we accept a 5% deviation from the expected probabilities

    def setUp(self) -> None:
        self.a_fr_locale = "fr_CA"
        self.a_en_locale = "en_CA"

    def test_whenPurchaseConditionFakerEN_thenAreInDeltaExpectedDefaultProb(
        self,
    ):
        quebec_purchase_condition_faker = QuebecPurchaseConditionFaker(locale=self.a_en_locale)

        purchase_conditions = [quebec_purchase_condition_faker.purchase_condition() == "used" for _ in range(self.n)]

        actual = sum(purchase_conditions) / self.n  # Prob of having a used car
        expected = 0.91

        self.assertAlmostEqual(actual, expected, delta=self.delta)

    def test_whenPurchaseConditionFakerFR_thenAreInDeltaExpectedDefaultProb(
        self,
    ):
        quebec_purchase_condition_faker = QuebecPurchaseConditionFaker(locale=self.a_fr_locale)

        purchase_conditions = [quebec_purchase_condition_faker.purchase_condition() == "usagé" for _ in range(self.n)]

        actual = sum(purchase_conditions) / self.n  # Prob of having a used car
        expected = 0.91

        self.assertAlmostEqual(actual, expected, delta=self.delta)

    def test_givenPurchaseConditionFakerFR_whenNewPurchaseCondition_thenReturnFRNew(self):
        quebec_purchase_condition_faker = QuebecPurchaseConditionFaker(locale=self.a_fr_locale)

        expected = "neuf"

        actual = quebec_purchase_condition_faker.new_purchase_condition()

        self.assertEqual(expected, actual)

    def test_givenPurchaseConditionFakerEN_whenNewPurchaseCondition_thenReturnENNew(self):
        quebec_purchase_condition_faker = QuebecPurchaseConditionFaker(locale=self.a_en_locale)

        expected = "new"

        actual = quebec_purchase_condition_faker.new_purchase_condition()

        self.assertEqual(expected, actual)



import os
from typing import Union

from ...faker.contract_faker.contract_faker import FPQ1ContractFaker
from ...faker.protections_faker.protections_faker import ProtectionsFaker
from ...model.tvae_synthetic_protections_model_wrapper import TVAESyntheticProtectionsModelWrapper


class TVAEFPQ1ContractFaker(FPQ1ContractFaker):
    def __init__(
        self, tvae_synthetic_data_faker_model_path: Union[None, str] = None, language: str = "en", seed: int = 42
    ) -> None:
        """
        A FPQ1 contract faker using a TVAE synthetic protections model to generate synthetic protections using CPU.


        tvae_synthetic_data_faker_model_path (str): Path to a TVAE model (see `here
            <https://docs.sdv.dev/sdv/single-table-data/modeling/synthesizers/tvaesynthesizer>`_
            for details on TVAE model) if you want to use your. If None, we will
            use our pre-trained model trained on a private dataset of a Canadian insurance company. By default,
            `"None"`.
        language (str): Either `'french'` (or `'fr'`) or `'english'` (or `'en'`). By default, `"en"`.
        seed (int): The seed value to use for the generation seed. By default, 42.
        """
        if tvae_synthetic_data_faker_model_path is None:
            tvae_synthetic_data_faker_model_path = (
                os.path.join(os.path.dirname(__file__), '../../resources', "tvae_model_q1_2023.pkl"),
            )
        protection_faker = ProtectionsFaker(
            model=TVAESyntheticProtectionsModelWrapper(tvae_synthetic_data_faker_model_path)
        )
        super().__init__(protection_faker, language, seed=seed)



from ..config import NEW_PAGE_TAG


def generate_bill_text(self):
    text = ""
    if self.payment_method == "facture":
        text += (
            "FACTURATION - Reçu officiel\n"
            "Assurance des particuliers\n"
            f"Numéro client : {self.client_number}\n"
            f"Nom et adresse de l'assuré\n"
            f"{self.insuree.name}\n"
            f"{self.insuree.address}\n"
            f"Nom du payeur : {self.insuree.name}\n"
            f"ÉTAT DE VOTRE COMPTE - MODE DE PAIEMENT COMPTANT SEULEMENT\n"
            f"CATÉGORIE D'ASSURANCE | N POLICE | TRANSACTION ACTUELLE | PRISE D'EFFET | TYPE DE TRANSACTION | "
            f"MONTANT INCLUANT TAXE | SOLDE DE LA POLICE | TOTAL\n"
            f"Automobile | 001 | | | SOLDE AVANT TRANSACTION | | {self.premium.total}$ | {self.premium.total}$\n"
            f"À payer-terme en vigueur {self.premium.total}$\n"
            f"{NEW_PAGE_TAG}\n"
        )

    return text



from ..config import (
    GENERAL_FAKE_PHONE_NUMBER,
    GENERAL_FAKE_PHONE_NUMBER_1_800,
    FAKE_FAX_PHONE_NUMBER,
    FAKE_EMAIL,
    CLAIMS_FAKE_PHONE_NUMBER,
    CLAIMS_FAKE_PHONE_NUMBER_1_800,
    FAKE_LEGAL_HELP_PHONE_NUMBER,
    FAKE_INSURER_ADDRESS,
    FAKE_INSURER_WEBSITE,
    NEW_PAGE_TAG,
)


def generate_header_text(self) -> str:
    content_text = (
        f"{self.formatted_text_date()}\n\n\n"
        f"ISSUE\n"
        f"Client N: {self.client_number}\n"
        f"Policy / Term:\n"
        f"{self.policy} / 12 month\n\n"
        f"ALWAYS BY YOUR SIDE FOR YOUR INSURANCE\n\n"
        f"Please find enclosed your insurance documents.\n"
        f"We recommend that you read them carefully to ensure that they are correct.\n"
        f"If an Important Instructions page is included with these documents, please read through it carefully "
        f"because it contains crucial information regarding your coverage.\n"
        f"Feel free to contact us if you have any questions or comments.\n\n"
        f"CONTACT US\n"
        f"Customer Service\n"
        f"{GENERAL_FAKE_PHONE_NUMBER}\n"
        f"{GENERAL_FAKE_PHONE_NUMBER_1_800}\n"
        f"Fax: {FAKE_FAX_PHONE_NUMBER}\n"
        f"{FAKE_EMAIL}\n"
        f"8 a.m. to 8 p.m.\n"
        f"Monday to Friday\n"
        f"8:30 a.m. to 4 p.m., Saturday\n"
        f"Claims\n"
        f"{CLAIMS_FAKE_PHONE_NUMBER}\n"
        f"{CLAIMS_FAKE_PHONE_NUMBER_1_800}\n"
        f"24/7 emergency\n"
        f"Legal Assistance\n"
        f"{FAKE_LEGAL_HELP_PHONE_NUMBER}\n"
        f"8:30 a.m. to 7 p.m.\n"
        f"Monday to Thursday\n"
        f"8:30 a.m. to 5 p.m., Friday\n"
        f"Your branch\n"
        f"{FAKE_INSURER_ADDRESS}\n"
        f"8:30 a.m. to 5 p.m.\n"
        f"Monday to Friday\n"
        f"{FAKE_INSURER_WEBSITE}\n"
    )
    if self.payment_method == "bill":
        # Case where payment is a bill, thus it include a billing section in the TDM
        content_text += (
            f"Your Insurance Documents\n"
            f"(Table of Contents)\n"
            f"Important Instructions\n"
            f"Automobile Insurance\n"
            f"Policy number: {self.policy}\n"
            f"{self.vehicle.format_vehicle_make_year_details()}\n"
            f"Insurance Certificates\n"
            f"(Documents to be kept on file)\n"
            f"Invoice\n"
            f"Client Advantages\n"
            f"(Description of your client benefits)\n"
            f"Wording and Endorsements\n"
            f"{NEW_PAGE_TAG}\n"
            f"{NEW_PAGE_TAG}\n"  # An empty page
        )
    else:
        # Case where payment is pre-authorized, thus no billing in the documents.
        content_text += (
            f"Your Insurance Documents\n"
            f"(Table of Contents)\n"
            f"Important Instructions\n"
            f"Automobile Insurance\n"
            f"Policy number: {self.policy}\n"
            f"{self.vehicle.format_vehicle_make_year_details()}\n"
            f"Insurance Certificates\n"
            f"(Documents to be kept on file)\n"
            f"Client Advantages\n"
            f"(Description of your client benefits)\n"
            f"Wording and Endorsements\n"
            f"{NEW_PAGE_TAG}\n"
            f"{NEW_PAGE_TAG}\n"  # An empty page
        )
    return content_text



import random
from typing import List


def liability_coverage_amount_faker(liability_amount_probs: List) -> int:
    """
    Faker function to generate a fake liability coverage base on probability vectors (2 elements). Liability amount can
     either be 1 million or 2 million dollars.

    Return:
        The liability coverage amount (int).
    """

    return random.choices([1000000, 2000000], weights=liability_amount_probs, k=1)[0]


def b1_deductible_faker() -> int:
    """
    Faker function to generate a fake B1 deductible.

    Return:
        The deductible amount (int).
    """
    return random.choices([250, 500, 1000], weights=[0.15, 0.7, 0.20], k=1)[0]


def b2_deductible_faker() -> int:
    """
    Faker function to generate a fake B2 deductible.

    Return:
        The deductible amount (int).
    """

    return random.choices([250, 500, 1000], weights=[0.20, 0.65, 0.15], k=1)[0]


def b3_deductible_faker() -> int:
    """
    Faker function to generate a fake B3 deductible.

    Return:
        The deductible amount (int).
    """

    return random.choices([250, 500, 1000], weights=[0.50, 0.40, 0.10], k=1)[0]


def b4_deductible_faker() -> int:
    """
    Faker function to generate a fake B4 deductible.
    """

    return random.choices([250, 500, 1000], weights=[0.55, 0.40, 0.05], k=1)[0]



from ..config import (
    GENERAL_FAKE_PHONE_NUMBER_1_800,
    NEW_PAGE_TAG,
    FAKE_FIDELITY_PROGRAM_NAME_FR,
    FAKE_ROADSIDE_ASSISTANCE_PHONE_NUMBER,
    FAKE_ROADSIDE_PROGRAM_NAME,
)


def generate_important_instructions_text(self) -> str:
    important_instructions = (
        f"INSTRUCTIONS IMPORTANTES\n\n"
        f"{self.formatted_text_date()}\n"
        f"Vous trouverez ci-dessous des instructions concernant des éléments très "
        f"importants de votre police d'assurance et nous vous recommandons de les "
        f"lire attentivement et d'y donner suite s'il y a lieu.\n\n"
    )

    fidelity = (
        f"Programme {FAKE_FIDELITY_PROGRAM_NAME_FR}\n\n"
        f"Parce que votre fidélité nous tient à coeur, nous sommes heureux de vous offrir des privilèges "
        f"qui s'accumulent en même temps que vos années de fidélité, grâce à notre programme "
        f"{FAKE_FIDELITY_PROGRAM_NAME_FR}. Voici le détail de vos privilèges, "
        f"que vous trouverez également dans votre Espace client.\n"
        f"Votre fidélité reconnue: Nouveau client\n"
        f"Vos privilèges:\n"
        f"\t Offres exclusives chez nos partenaires\n"
        f"\t Assistance juridique\n"
        f"\t Service d'aide psychologique\n"
    )

    if self.protections.is_protected("FAQ41Coverage"):
        fidelity += "\t Congé de franchise en cas de perte totale ou de délit de fuite (auto)\n"

    fidelity += (
        "Cette liste représente l'ensemble des privilèges associés à votre fidélité reconnue. "
        "Veuillez noter que les nouveaux privilèges seront disponibles au "
        "moment du renouvellement de chacune des polices d'assurance détenues par un "
        "membre de votre maisonnée. Référez-vous aux Conditions "
        "particulières de vos divers contrats d'assurance détenus chez "
        "nous pour connaître le détail des protections applicables.\n"
    )

    if (
        self.protections.is_protected("FAQ33Coverage")
        or self.protections.is_protected("FAQ5ACoverage")
        or self.protections.is_protected("FAQ23ACoverage")
    ):
        notice_to_the_insuree = "Avis à l'assuré\n"
    else:
        notice_to_the_insuree = ""

    if self.protections.is_protected("FAQ33Coverage"):
        notice_to_the_insuree += (
            f"POLICE AUTOMOBILE N {self.policy_number}\n"
            f"Vous avez opté pour {FAKE_ROADSIDE_PROGRAM_NAME} assistance routière, qui vous procure des services de "
            f"dépannage partout au Canada et aux États-Unis, 24 heures sur 24, "
            f"365 jours par année.\n"
            f"Ainsi, trois semaines après la date de prise d'effet de l'avenant F.A.Q. N 33 "
            f"Assurance des frais de dépannage, vous recevrez une trousse pour "
            f"chaque véhicule admissible, contenant entre autres votre carte d'adhérent ainsi "
            f"que votre guide de l'usager.\n"
            f"Dès que l'avenant F.A.Q. N 33 entrera en vigueur, vous pourrez bénéficier de ce "
            f"service même si vous n'avez pas reçu votre trousse {FAKE_ROADSIDE_PROGRAM_NAME} "
            f"assistance routière. En cas de besoin, il vous suffira de composer le "
            f"{FAKE_ROADSIDE_ASSISTANCE_PHONE_NUMBER} (24 heures sur 24, 7 jours sur 7) et de mentionner le numéro de "
            f"police du véhicule couvert au préposé.\n"
        )

    if self.protections.is_protected("FAQ5ACoverage") or self.protections.is_protected("FAQ23ACoverage"):
        if self.protections.is_protected("FAQ5ACoverage"):
            creditor_type = "locateur"
        else:
            creditor_type = "créancier"
        notice_to_the_insuree += (
            f"POLICE AUTOMOBILE N {self.policy_number} - Véhicule 1 : "
            f"{self.vehicle.format_vehicle_make_model_year_details()}\n"
            f"Nous joignons une copie de votre police à remettre à votre {creditor_type}, "
            f"s'il la demande. Elle se trouve à la toute fin du présent envoi.\n"
        )

    actions_to_take = (
        f"\nActions à entreprendre\n\n"
        f"Assurance auto\n"
        f"\t Détachez et conservez avec vous vos certificats d'assurance.\n"
        f"\t Détachez et conservez avec votre permis de conduire le certificat "
        f"d'assurance pour location de voiture. Vous éviterez ainsi de débourser les frais "
        f"d'assurance additionnels auprès de l'agence de location.\n"
        f"\t Veuillez prendre connaissance de la section Conditions particulières de votre police "
        f"d'assurance automobile du Québec, afin de vous assurer que vos couvertures répondent "
        f"bien à vos besoins.\n"
        f"\t N'hésitez pas à joindre le service à la clientèle au "
        f"{GENERAL_FAKE_PHONE_NUMBER_1_800} pour toute question relative à votre contrat.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    text = important_instructions + fidelity + notice_to_the_insuree + NEW_PAGE_TAG + "\n" + actions_to_take

    return text



# pylint: disable=too-many-locals


from risc_generator import Protections, FPQ1Contract
from tests.domain.contract.contract_base import ContractTestBase

include = "include"


class ContractCertificatesTest(ContractTestBase):
    def setUp(self) -> None:
        self.maxDiff = None

    def test_givenAFRContract_whenGenerateCertificatesText_thenReturnProperFRText(self):
        a_protections = Protections(protections=self.a_dict_of_protections.copy())

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_certificates_text()

        a_start_date = "2022-02-01"  # See self.a_contract_start_date
        a_contract_end_date = "2023-02-01"  # 1 year later
        a_claims_phone_number = "098 765-4321"
        a_claims_phone_number_1_800 = "1 800 765-4321"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_policy_number = "1234567-001"
        a_insuree_name = "A Name"
        a_insuree_address = "An address\nA city, QC, A1A 1A1"
        an_insured_car = "AMaker AModel 2020 1234567890"
        a_website = "assureur.ca"
        a_insurer_name = "Assureur"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Les certiﬁcats d'assurance du véhicule assuré peuvent maintenant être téléchargés dans l'application "
            "Wallet de Apple. Détails au verso.\n"
            f"VOICI VOS DEUX CERTIFICATS D'ASSURANCE POUR LE VÉHICULE ASSURÉ\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{a_policy_number}\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
            f"{an_insured_car}\n"
            f"DURÉE DU CONTRAT DU "
            f"{a_start_date} AU {a_contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"Vous voulez déclarer un sinistre?\n"
            f"Communiquez avec nous.\n"
            f"Service des sinistres 7 jours/24h :\n"
            f"Local: {a_claims_phone_number}\n"
            f"Sans frais : {a_claims_phone_number_1_800}\n"
            f"Service à la clientèle\n"
            f"Sans frais :{a_general_phone_number_1_800}\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{a_policy_number}\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
            f"{an_insured_car}\n"
            f"DURÉE DU CONTRAT DU "
            f"{a_start_date} AU {a_contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"Vous voulez déclarer un sinistre?\n"
            f"Communiquez avec nous.\n"
            f"Service des sinistres 7 jours/24h :\n"
            f"Local: {a_claims_phone_number}\n"
            f"Sans frais : {a_claims_phone_number_1_800}\n"
            f"Service à la clientèle\n"
            f"Sans frais :{a_general_phone_number_1_800}\n\n"
            f"{a_new_page}\n"
            "CERTIFICATS D'ASSURANCE RESPONABILITÉ AUTOMOBILE\n"
            "CONSERVER AVEC VOTRE PERMIS DE CONDUIRE\n"
            f"Pour télécharger vos certiﬁcats dans l'application Wallet (disponible seulement sur iPhone) :\n"
            f"\t1. Connectez-vous à Espace client à partir de votre iPhone, au {a_website}/espaceclient.\n"
            f"\t2. Dans la section Auto, habitation et véhicules de loisirs, cliquez sur « Obtenir "
            f"un certiﬁcat d'assurance », puis sur « Ajouter à Apple Wallet ».\n"
            f"Vous devrez mettre à jour vos certiﬁcats à chaque renouvellement ou modiﬁcation de votre police "
            f"d'assurance en les téléchargeant de nouveau dans l'application Wallet.\n"
            f"{a_insurer_name} ne peut pas garantir l'acceptation des certiﬁcats en format électronique par "
            f"tous les corps policiers. Assurez-vous de toujours en garder une version papier avec vous.\n\n"
            "CERTIFICAT D'ASSURANCE RESPONSABILITÉ AUTOMOBILE\n"
            "Le présent certiﬁcat est assujetti aux dispositions et conditions de la police "
            "d'assurance automobile de l'Assureur.\n"
            "Ce certiﬁcat atteste que la personne susnommée est assurée contre la responsabilité "
            "pour blessures et dommages aux biens découlant de l'usage du véhicule ci-décrit, "
            "conformément aux limites minimales exigées par les lois d'assurance en vigueur dans "
            "chacune des provinces canadiennes.\n"
            "AVERTISSEMENT - Quiconque émet ou présente un tel certiﬁcat comme preuve d'une "
            "police d'assurance responsabilité qui effectivement n'est pas en vigueur, est coupable "
            "d'une infraction passible d'une forte amende et/ou d'emprisonnement et suspension "
            "de son permis.\n"
            "Tout sinistre devrait être rapporté immédiatement.\n\n"
            "CANADA INTER-PROVINCE MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
            "This certiﬁcate is subject to the terms and conditions of the Insurers standard "
            "automobile policy.\n"
            "This certiﬁes that the party named herein is insured against liability for bodily injury and "
            "property damage by reason of the operation of the motor vehicle described herein, for an "
            "amount not less than the statutory minimum requirements of every province of Canada.\n"
            "WARNING - Any person who issues or presents such a certiﬁcate as proof of insurance "
            "when no policy exists is guilty of fraud and liable to a heavy ﬁ ne and/or imprisonment. "
            "His driving permit may also be suspended.\n"
            "All loss should be reported immediately.\n\n"
            "CERTIFICAT D'ASSURANCE RESPONSABILITÉ AUTOMOBILE\n"
            "Le présent certiﬁcat est assujetti aux dispositions et conditions de la police "
            "d'assurance automobile de l'Assureur.\n"
            "Ce certiﬁcat atteste que la personne susnommée est assurée contre la responsabilité "
            "pour blessures et dommages aux biens découlant de l'usage du véhicule ci-décrit, "
            "conformément aux limites minimales exigées par les lois d'assurance en vigueur dans "
            "chacune des provinces canadiennes.\n"
            "AVERTISSEMENT - Quiconque émet ou présente un tel certiﬁcat comme preuve d'une "
            "police d'assurance responsabilité qui effectivement n'est pas en vigueur, est coupable "
            "d'une infraction passible d'une forte amende et/ou d'emprisonnement et suspension "
            "de son permis.\n"
            "Tout sinistre devrait être rapporté immédiatement.\n\n"
            "CANADA INTER-PROVINCE MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
            "This certiﬁcate is subject to the terms and conditions of the Insurers standard "
            "automobile policy.\n"
            "This certiﬁes that the party named herein is insured against liability for bodily injury and "
            "property damage by reason of the operation of the motor vehicle described herein, for an "
            "amount not less than the statutory minimum requirements of every province of Canada.\n"
            "WARNING - Any person who issues or presents such a certiﬁcate as proof of insurance "
            "when no policy exists is guilty of fraud and liable to a heavy ﬁ ne and/or imprisonment. "
            "His driving permit may also be suspended.\n"
            "All loss should be reported immediately.\n\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithFAQ27_whenGenerateCertificatesText_thenReturnProperFRText(self):
        protections = self.a_dict_of_protections.copy()
        protections.update({"FAQ27Coverage": include})
        a_protections = Protections(protections=protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_certificates_text()

        a_start_date = "2022-02-01"  # See self.a_contract_start_date
        a_contract_end_date = "2023-02-01"  # 1 year later
        a_claims_phone_number = "098 765-4321"
        a_claims_phone_number_1_800 = "1 800 765-4321"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_policy_number = "1234567-001"
        a_insuree_name = "A Name"
        a_insuree_address = "An address\nA city, QC, A1A 1A1"
        an_insured_car = "AMaker AModel 2020 1234567890"
        a_website = "assureur.ca"
        a_insurer_name = "Assureur"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Les certiﬁcats d'assurance du véhicule assuré peuvent maintenant être téléchargés dans l'application "
            "Wallet de Apple. Détails au verso.\n"
            f"F.A.Q. N 27 : VOTRE CERTIFICAT POUR VÉHICULE DE LOCATION\n"
            f"Lors de votre prochaine location d'auto, présentez ce certiﬁcat au locateur pour "
            f"éviter des frais additionnels d'assurance. Il est valide au Canada et aux États-Unis "
            f"jusqu'à concurrence de Détails au verso du certiﬁcat.\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{a_policy_number}\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"DURÉE DU CONTRAT DU "
            f"{a_start_date} AU {a_contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"IMPORTANT\n"
            f"AU BESOIN, VEUILLEZ PRÉSENTER CE CERTIFICAT À VOTRE AGENCE DE LOCATION DE VOITURES\n"
            f"IF NECESSARY, PLEASE PRESENT THIS CERTIFICATE TO YOUR AUTOMOBILE RENTAL AGENCY\n"
            f"Vous voulez déclarer un sinistre?\n"
            f"Communiquez avec nous.\n"
            f"Service des sinistres 7 jours/24h :\n"
            f"Local: {a_claims_phone_number}\n"
            f"Sans frais : {a_claims_phone_number_1_800}\n"
            f"Service à la clientèle\n"
            f"Sans frais :{a_general_phone_number_1_800}\n\n"
            f"VOICI VOS DEUX CERTIFICATS D'ASSURANCE POUR LE VÉHICULE ASSURÉ\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{a_policy_number}\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
            f"{an_insured_car}\n"
            f"DURÉE DU CONTRAT DU "
            f"{a_start_date} AU {a_contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"Vous voulez déclarer un sinistre?\n"
            f"Communiquez avec nous.\n"
            f"Service des sinistres 7 jours/24h :\n"
            f"Local: {a_claims_phone_number}\n"
            f"Sans frais : {a_claims_phone_number_1_800}\n"
            f"Service à la clientèle\n"
            f"Sans frais :{a_general_phone_number_1_800}\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{a_policy_number}\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
            f"{an_insured_car}\n"
            f"DURÉE DU CONTRAT DU "
            f"{a_start_date} AU {a_contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"Vous voulez déclarer un sinistre?\n"
            f"Communiquez avec nous.\n"
            f"Service des sinistres 7 jours/24h :\n"
            f"Local: {a_claims_phone_number}\n"
            f"Sans frais : {a_claims_phone_number_1_800}\n"
            f"Service à la clientèle\n"
            f"Sans frais :{a_general_phone_number_1_800}\n\n"
            f"{a_new_page}\n"
            "CERTIFICATS D'ASSURANCE RESPONABILITÉ AUTOMOBILE\n"
            "CONSERVER AVEC VOTRE PERMIS DE CONDUIRE\n"
            f"Pour télécharger vos certiﬁcats dans l'application Wallet (disponible seulement sur iPhone) :\n"
            f"\t1. Connectez-vous à Espace client à partir de votre iPhone, au {a_website}/espaceclient.\n"
            f"\t2. Dans la section Auto, habitation et véhicules de loisirs, cliquez sur « Obtenir "
            f"un certiﬁcat d'assurance », puis sur « Ajouter à Apple Wallet ».\n"
            f"Vous devrez mettre à jour vos certiﬁcats à chaque renouvellement ou modiﬁcation de votre police "
            f"d'assurance en les téléchargeant de nouveau dans l'application Wallet.\n"
            f"{a_insurer_name} ne peut pas garantir l'acceptation des certiﬁcats en format électronique par "
            f"tous les corps policiers. Assurez-vous de toujours en garder une version papier avec vous.\n\n"
            f"CERTIFICAT D'ASSURANCE POUR LOCATION DE VOITURE\n"
            f"(Valide au Canada et aux États-Unis d'Amérique seulement)\n"
            f"Application particulière de l'avenant F.A.Q. 27 : Responsabilité civile du fait de dommages causés à "
            f"des véhicules dont l'assuré désigné n'est pas propriétaire (incluant les véhicules fournis par un "
            f"employeur).\n"
            f"En cas de vol ou d'endommagement d'un véhicule de location, {a_insurer_name} s'engage "
            f"à rembourser, à l'Assuré ou au propriétaire du véhicule, le montant des dommages réellement encourus "
            f"et dont l'Assuré ou son conjoint s'est rendu responsable en vertu d'un contrat de location, le tout "
            f"jusqu'à concurrence de : 75 000 $\n\n"
            f"CAR RENTAL INSURANCE CERTIFICATE\n"
            f"(Valid only in Canada and the United States of America)\n"
            f"Speciﬁc application of Q.E.F. endorsement B 27 : Civil liability resulting from damage caused to "
            f"vehicles of which named insured is not owner (including vehicles provided by an employer).\n"
            f"In the event of theft or damage to a rental vehicle, {a_insurer_name} "
            f"agrees to refund the Insured or the owner of the vehicle, the amount of damage actually "
            f"incurred and for which the Insured or his/her spouse assumes liability under a rental "
            f"contract, in all up to: $75,000\n"
            "CERTIFICAT D'ASSURANCE RESPONSABILITÉ AUTOMOBILE\n"
            "Le présent certiﬁcat est assujetti aux dispositions et conditions de la police "
            "d'assurance automobile de l'Assureur.\n"
            "Ce certiﬁcat atteste que la personne susnommée est assurée contre la responsabilité "
            "pour blessures et dommages aux biens découlant de l'usage du véhicule ci-décrit, "
            "conformément aux limites minimales exigées par les lois d'assurance en vigueur dans "
            "chacune des provinces canadiennes.\n"
            "AVERTISSEMENT - Quiconque émet ou présente un tel certiﬁcat comme preuve d'une "
            "police d'assurance responsabilité qui effectivement n'est pas en vigueur, est coupable "
            "d'une infraction passible d'une forte amende et/ou d'emprisonnement et suspension "
            "de son permis.\n"
            "Tout sinistre devrait être rapporté immédiatement.\n\n"
            "CANADA INTER-PROVINCE MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
            "This certiﬁcate is subject to the terms and conditions of the Insurers standard "
            "automobile policy.\n"
            "This certiﬁes that the party named herein is insured against liability for bodily injury and "
            "property damage by reason of the operation of the motor vehicle described herein, for an "
            "amount not less than the statutory minimum requirements of every province of Canada.\n"
            "WARNING - Any person who issues or presents such a certiﬁcate as proof of insurance "
            "when no policy exists is guilty of fraud and liable to a heavy ﬁ ne and/or imprisonment. "
            "His driving permit may also be suspended.\n"
            "All loss should be reported immediately.\n\n"
            "CERTIFICAT D'ASSURANCE RESPONSABILITÉ AUTOMOBILE\n"
            "Le présent certiﬁcat est assujetti aux dispositions et conditions de la police "
            "d'assurance automobile de l'Assureur.\n"
            "Ce certiﬁcat atteste que la personne susnommée est assurée contre la responsabilité "
            "pour blessures et dommages aux biens découlant de l'usage du véhicule ci-décrit, "
            "conformément aux limites minimales exigées par les lois d'assurance en vigueur dans "
            "chacune des provinces canadiennes.\n"
            "AVERTISSEMENT - Quiconque émet ou présente un tel certiﬁcat comme preuve d'une "
            "police d'assurance responsabilité qui effectivement n'est pas en vigueur, est coupable "
            "d'une infraction passible d'une forte amende et/ou d'emprisonnement et suspension "
            "de son permis.\n"
            "Tout sinistre devrait être rapporté immédiatement.\n\n"
            "CANADA INTER-PROVINCE MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
            "This certiﬁcate is subject to the terms and conditions of the Insurers standard "
            "automobile policy.\n"
            "This certiﬁes that the party named herein is insured against liability for bodily injury and "
            "property damage by reason of the operation of the motor vehicle described herein, for an "
            "amount not less than the statutory minimum requirements of every province of Canada.\n"
            "WARNING - Any person who issues or presents such a certiﬁcate as proof of insurance "
            "when no policy exists is guilty of fraud and liable to a heavy ﬁ ne and/or imprisonment. "
            "His driving permit may also be suspended.\n"
            "All loss should be reported immediately.\n\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContract_whenGenerateCertificatesText_thenReturnProperENText(self):
        a_protections = Protections(protections=self.a_dict_of_protections.copy())

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_certificates_text()

        a_start_date = "2022-02-01"  # See self.a_contract_start_date
        a_contract_end_date = "2023-02-01"  # 1 year later
        a_claims_phone_number = "098 765-4321"
        a_claims_phone_number_1_800 = "1 800 765-4321"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_policy_number = "1234567-001"
        a_insuree_name = "A Name"
        a_insuree_address = "An address\nA city, QC, A1A 1A1"
        an_insured_car = "AMaker AModel 2020 1234567890"
        a_website = "assureur.ca"
        a_insurer_name = "Assureur"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "The insured vehicle's insurance certificates can now be downloaded into the Apple Wallet application. "
            "Details on the back.\n"
            f"HERE ARE YOUR TWO INSURANCE CERTIFICATES FOR THE INSURED VEHICLE\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{a_policy_number}\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
            f"{an_insured_car}\n"
            f"DURÉE DU CONTRAT DU "
            f"{a_start_date} AU {a_contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"Need to file an insurance claim?\n"
            f"Contact us.\n"
            f"Claims Department open 24/7\n"
            f"Local: {a_claims_phone_number}\n"
            f"Toll free: {a_claims_phone_number_1_800}\n"
            f"Customer Service\n"
            f"Toll free:{a_general_phone_number_1_800}\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{a_policy_number}\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
            f"{an_insured_car}\n"
            f"DURÉE DU CONTRAT DU "
            f"{a_start_date} AU {a_contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"Need to file an insurance claim?\n"
            f"Contact us.\n"
            f"Claims Department open 24/7\n"
            f"Local: {a_claims_phone_number}\n"
            f"Toll free: {a_claims_phone_number_1_800}\n"
            f"Customer Service\n"
            f"Toll free:{a_general_phone_number_1_800}\n\n"
            f"{a_new_page}\n"
            "CANADA INTER-PROVINCE\n"
            "MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
            "Keep with your driver's licence\n"
            f"To download your certificate into the Apple Wallet application (available on iPhone only):\n"
            f"\t1. Log in to the Client Centre from your phone at {a_website}/clientcentre.\n"
            f"\t2. In the Home, auto and leisure vehicles section, click on ''Get an insurance certificate'', then"
            f"''Add to Apple Wallet''.\n"
            f"You will have to update your certificates at each renewal of, or change to, your insurance policy by "
            f"downloading it again to the Apple Wallet application.\n"
            f"{a_insurer_name} cannot guarantee that the electronic version of the certificates will be accepted by "
            f"all police forces. Make sure you always have a paper version on hand.\n\n"
            "CERTIFICAT D'ASSURANCE RESPONSABILITÉ AUTOMOBILE\n"
            "Le présent certiﬁcat est assujetti aux dispositions et conditions de la police "
            "d'assurance automobile de l'Assureur.\n"
            "Ce certiﬁcat atteste que la personne susnommée est assurée contre la responsabilité "
            "pour blessures et dommages aux biens découlant de l'usage du véhicule ci-décrit, "
            "conformément aux limites minimales exigées par les lois d'assurance en vigueur dans "
            "chacune des provinces canadiennes.\n"
            "AVERTISSEMENT - Quiconque émet ou présente un tel certiﬁcat comme preuve d'une "
            "police d'assurance responsabilité qui effectivement n'est pas en vigueur, est coupable "
            "d'une infraction passible d'une forte amende et/ou d'emprisonnement et suspension "
            "de son permis.\n"
            "Tout sinistre devrait être rapporté immédiatement.\n\n"
            "CANADA INTER-PROVINCE MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
            "This certiﬁcate is subject to the terms and conditions of the Insurers standard "
            "automobile policy.\n"
            "This certiﬁes that the party named herein is insured against liability for bodily injury and "
            "property damage by reason of the operation of the motor vehicle described herein, for an "
            "amount not less than the statutory minimum requirements of every province of Canada.\n"
            "WARNING - Any person who issues or presents such a certiﬁcate as proof of insurance "
            "when no policy exists is guilty of fraud and liable to a heavy ﬁ ne and/or imprisonment. "
            "His driving permit may also be suspended.\n"
            "All loss should be reported immediately.\n\n"
            "CERTIFICAT D'ASSURANCE RESPONSABILITÉ AUTOMOBILE\n"
            "Le présent certiﬁcat est assujetti aux dispositions et conditions de la police "
            "d'assurance automobile de l'Assureur.\n"
            "Ce certiﬁcat atteste que la personne susnommée est assurée contre la responsabilité "
            "pour blessures et dommages aux biens découlant de l'usage du véhicule ci-décrit, "
            "conformément aux limites minimales exigées par les lois d'assurance en vigueur dans "
            "chacune des provinces canadiennes.\n"
            "AVERTISSEMENT - Quiconque émet ou présente un tel certiﬁcat comme preuve d'une "
            "police d'assurance responsabilité qui effectivement n'est pas en vigueur, est coupable "
            "d'une infraction passible d'une forte amende et/ou d'emprisonnement et suspension "
            "de son permis.\n"
            "Tout sinistre devrait être rapporté immédiatement.\n\n"
            "CANADA INTER-PROVINCE MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
            "This certiﬁcate is subject to the terms and conditions of the Insurers standard "
            "automobile policy.\n"
            "This certiﬁes that the party named herein is insured against liability for bodily injury and "
            "property damage by reason of the operation of the motor vehicle described herein, for an "
            "amount not less than the statutory minimum requirements of every province of Canada.\n"
            "WARNING - Any person who issues or presents such a certiﬁcate as proof of insurance "
            "when no policy exists is guilty of fraud and liable to a heavy ﬁ ne and/or imprisonment. "
            "His driving permit may also be suspended.\n"
            "All loss should be reported immediately.\n\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithFAQ27_whenGenerateCertificatesText_thenReturnProperENText(self):
        protections = self.a_dict_of_protections.copy()
        protections.update({"FAQ27Coverage": include})
        a_protections = Protections(protections=protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_certificates_text()

        a_start_date = "2022-02-01"  # See self.a_contract_start_date
        a_contract_end_date = "2023-02-01"  # 1 year later
        a_claims_phone_number = "098 765-4321"
        a_claims_phone_number_1_800 = "1 800 765-4321"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_policy_number = "1234567-001"
        a_insuree_name = "A Name"
        a_insuree_address = "An address\nA city, QC, A1A 1A1"
        an_insured_car = "AMaker AModel 2020 1234567890"
        a_website = "assureur.ca"
        a_insurer_name = "Assureur"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "The insured vehicle's insurance certificates can now be downloaded into the Apple Wallet application. "
            "Details on the back.\n"
            f"Q.E.F. N 27 : YOUR RENTAL VEHICLE INSURANCE CERTIFICATE\n"
            f"The next time you rent a car, show this certificate to the rental agent to avoid paying additional "
            f"insurance fees. It's valid in Canada and the United States up to a maximum of "
            f"Details on back of the certificate.\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{a_policy_number}\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"DURÉE DU CONTRAT DU "
            f"{a_start_date} AU {a_contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"IMPORTANT\n"
            f"AU BESOIN, VEUILLEZ PRÉSENTER CE CERTIFICAT À VOTRE AGENCE DE LOCATION DE VOITURES\n"
            f"IF NECESSARY, PLEASE PRESENT THIS CERTIFICATE TO YOUR AUTOMOBILE RENTAL AGENCY\n"
            f"Need to file an insurance claim?\n"
            f"Contact us.\n"
            f"Claims Department open 24/7\n"
            f"Local: {a_claims_phone_number}\n"
            f"Toll free: {a_claims_phone_number_1_800}\n"
            f"Customer Service\n"
            f"Toll free:{a_general_phone_number_1_800}\n\n"
            f"HERE ARE YOUR TWO INSURANCE CERTIFICATES FOR THE INSURED VEHICLE\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{a_policy_number}\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
            f"{an_insured_car}\n"
            f"DURÉE DU CONTRAT DU "
            f"{a_start_date} AU {a_contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"Need to file an insurance claim?\n"
            f"Contact us.\n"
            f"Claims Department open 24/7\n"
            f"Local: {a_claims_phone_number}\n"
            f"Toll free: {a_claims_phone_number_1_800}\n"
            f"Customer Service\n"
            f"Toll free:{a_general_phone_number_1_800}\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{a_policy_number}\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
            f"{an_insured_car}\n"
            f"DURÉE DU CONTRAT DU "
            f"{a_start_date} AU {a_contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"Need to file an insurance claim?\n"
            f"Contact us.\n"
            f"Claims Department open 24/7\n"
            f"Local: {a_claims_phone_number}\n"
            f"Toll free: {a_claims_phone_number_1_800}\n"
            f"Customer Service\n"
            f"Toll free:{a_general_phone_number_1_800}\n\n"
            f"{a_new_page}\n"
            "CANADA INTER-PROVINCE\n"
            "MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
            "Keep with your driver's licence\n"
            f"To download your certificate into the Apple Wallet application (available on iPhone only):\n"
            f"\t1. Log in to the Client Centre from your phone at {a_website}/clientcentre.\n"
            f"\t2. In the Home, auto and leisure vehicles section, click on ''Get an insurance certificate'', then"
            f"''Add to Apple Wallet''.\n"
            f"You will have to update your certificates at each renewal of, or change to, your insurance policy by "
            f"downloading it again to the Apple Wallet application.\n"
            f"{a_insurer_name} cannot guarantee that the electronic version of the certificates will be accepted by "
            f"all police forces. Make sure you always have a paper version on hand.\n\n"
            f"CERTIFICAT D'ASSURANCE POUR LOCATION DE VOITURE\n"
            f"(Valide au Canada et aux États-Unis d'Amérique seulement)\n"
            f"Application particulière de l'avenant F.A.Q. 27 : Responsabilité civile du fait de dommages causés à "
            f"des véhicules dont l'assuré désigné n'est pas propriétaire (incluant les véhicules fournis par un "
            f"employeur).\n"
            f"En cas de vol ou d'endommagement d'un véhicule de location, {a_insurer_name} s'engage "
            f"à rembourser, à l'Assuré ou au propriétaire du véhicule, le montant des dommages réellement encourus "
            f"et dont l'Assuré ou son conjoint s'est rendu responsable en vertu d'un contrat de location, le tout "
            f"jusqu'à concurrence de : 75 000 $\n\n"
            f"CAR RENTAL INSURANCE CERTIFICATE\n"
            f"(Valid only in Canada and the United States of America)\n"
            f"Speciﬁc application of Q.E.F. endorsement B 27 : Civil liability resulting from damage caused to "
            f"vehicles of which named insured is not owner (including vehicles provided by an employer).\n"
            f"In the event of theft or damage to a rental vehicle, {a_insurer_name} "
            f"agrees to refund the Insured or the owner of the vehicle, the amount of damage actually "
            f"incurred and for which the Insured or his/her spouse assumes liability under a rental "
            f"contract, in all up to: $75,000\n"
            "CERTIFICAT D'ASSURANCE RESPONSABILITÉ AUTOMOBILE\n"
            "Le présent certiﬁcat est assujetti aux dispositions et conditions de la police "
            "d'assurance automobile de l'Assureur.\n"
            "Ce certiﬁcat atteste que la personne susnommée est assurée contre la responsabilité "
            "pour blessures et dommages aux biens découlant de l'usage du véhicule ci-décrit, "
            "conformément aux limites minimales exigées par les lois d'assurance en vigueur dans "
            "chacune des provinces canadiennes.\n"
            "AVERTISSEMENT - Quiconque émet ou présente un tel certiﬁcat comme preuve d'une "
            "police d'assurance responsabilité qui effectivement n'est pas en vigueur, est coupable "
            "d'une infraction passible d'une forte amende et/ou d'emprisonnement et suspension "
            "de son permis.\n"
            "Tout sinistre devrait être rapporté immédiatement.\n\n"
            "CANADA INTER-PROVINCE MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
            "This certiﬁcate is subject to the terms and conditions of the Insurers standard "
            "automobile policy.\n"
            "This certiﬁes that the party named herein is insured against liability for bodily injury and "
            "property damage by reason of the operation of the motor vehicle described herein, for an "
            "amount not less than the statutory minimum requirements of every province of Canada.\n"
            "WARNING - Any person who issues or presents such a certiﬁcate as proof of insurance "
            "when no policy exists is guilty of fraud and liable to a heavy ﬁ ne and/or imprisonment. "
            "His driving permit may also be suspended.\n"
            "All loss should be reported immediately.\n\n"
            "CERTIFICAT D'ASSURANCE RESPONSABILITÉ AUTOMOBILE\n"
            "Le présent certiﬁcat est assujetti aux dispositions et conditions de la police "
            "d'assurance automobile de l'Assureur.\n"
            "Ce certiﬁcat atteste que la personne susnommée est assurée contre la responsabilité "
            "pour blessures et dommages aux biens découlant de l'usage du véhicule ci-décrit, "
            "conformément aux limites minimales exigées par les lois d'assurance en vigueur dans "
            "chacune des provinces canadiennes.\n"
            "AVERTISSEMENT - Quiconque émet ou présente un tel certiﬁcat comme preuve d'une "
            "police d'assurance responsabilité qui effectivement n'est pas en vigueur, est coupable "
            "d'une infraction passible d'une forte amende et/ou d'emprisonnement et suspension "
            "de son permis.\n"
            "Tout sinistre devrait être rapporté immédiatement.\n\n"
            "CANADA INTER-PROVINCE MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
            "This certiﬁcate is subject to the terms and conditions of the Insurers standard "
            "automobile policy.\n"
            "This certiﬁes that the party named herein is insured against liability for bodily injury and "
            "property damage by reason of the operation of the motor vehicle described herein, for an "
            "amount not less than the statutory minimum requirements of every province of Canada.\n"
            "WARNING - Any person who issues or presents such a certiﬁcate as proof of insurance "
            "when no policy exists is guilty of fraud and liable to a heavy ﬁ ne and/or imprisonment. "
            "His driving permit may also be suspended.\n"
            "All loss should be reported immediately.\n\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)



from ..config import (
    CLAIMS_FAKE_PHONE_NUMBER,
    CLAIMS_FAKE_PHONE_NUMBER_1_800,
    GENERAL_FAKE_PHONE_NUMBER_1_800,
    FAKE_INSURER_WEBSITE,
    FAKE_INSURER_NAME,
    NEW_PAGE_TAG,
)
from ..fr_en.back_of_certificates import back_of_certificates_text


def generate_certificates_text(self) -> str:
    text = (
        "Les certiﬁcats d'assurance du véhicule assuré peuvent maintenant être téléchargés dans l'application "
        "Wallet de Apple. Détails au verso.\n"
    )

    if self.protections.is_protected("FAQ27Coverage") or self.protections.is_protected("FAQ27aCoverage"):
        text += (
            f"F.A.Q. N 27 : VOTRE CERTIFICAT POUR VÉHICULE DE LOCATION\n"
            f"Lors de votre prochaine location d'auto, présentez ce certiﬁcat au locateur pour "
            f"éviter des frais additionnels d'assurance. Il est valide au Canada et aux États-Unis "
            f"jusqu'à concurrence de Détails au verso du certiﬁcat.\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{self.policy_number}\n"
            f"{self.insuree.name}\n"
            f"{self.insuree.address}\n"
            f"DURÉE DU CONTRAT DU "
            f"{self.contract_start_date} AU {self.contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"IMPORTANT\n"
            f"AU BESOIN, VEUILLEZ PRÉSENTER CE CERTIFICAT À VOTRE AGENCE DE LOCATION DE VOITURES\n"
            f"IF NECESSARY, PLEASE PRESENT THIS CERTIFICATE TO YOUR AUTOMOBILE RENTAL AGENCY\n"
            f"Vous voulez déclarer un sinistre?\n"
            f"Communiquez avec nous.\n"
            f"Service des sinistres 7 jours/24h :\n"
            f"Local: {CLAIMS_FAKE_PHONE_NUMBER}\n"
            f"Sans frais : {CLAIMS_FAKE_PHONE_NUMBER_1_800}\n"
            f"Service à la clientèle\n"
            f"Sans frais :{GENERAL_FAKE_PHONE_NUMBER_1_800}\n\n"
        )

    text += (
        f"VOICI VOS DEUX CERTIFICATS D'ASSURANCE POUR LE VÉHICULE ASSURÉ\n"
        f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
        f"{self.policy_number}\n"
        f"{self.insuree.name}\n"
        f"{self.insuree.address}\n"
        f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
        f"{self.vehicle.format_vehicle_complete_details()}\n"
        f"DURÉE DU CONTRAT DU "
        f"{self.contract_start_date} AU {self.contract_end_date}\n"
        f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
        f"Vous voulez déclarer un sinistre?\n"
        f"Communiquez avec nous.\n"
        f"Service des sinistres 7 jours/24h :\n"
        f"Local: {CLAIMS_FAKE_PHONE_NUMBER}\n"
        f"Sans frais : {CLAIMS_FAKE_PHONE_NUMBER_1_800}\n"
        f"Service à la clientèle\n"
        f"Sans frais :{GENERAL_FAKE_PHONE_NUMBER_1_800}\n"
        f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
        f"{self.policy_number}\n"
        f"{self.insuree.name}\n"
        f"{self.insuree.address}\n"
        f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
        f"{self.vehicle.format_vehicle_complete_details()}\n"
        f"DURÉE DU CONTRAT DU "
        f"{self.contract_start_date} AU {self.contract_end_date}\n"
        f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
        f"Vous voulez déclarer un sinistre?\n"
        f"Communiquez avec nous.\n"
        f"Service des sinistres 7 jours/24h :\n"
        f"Local: {CLAIMS_FAKE_PHONE_NUMBER}\n"
        f"Sans frais : {CLAIMS_FAKE_PHONE_NUMBER_1_800}\n"
        f"Service à la clientèle\n"
        f"Sans frais :{GENERAL_FAKE_PHONE_NUMBER_1_800}\n\n"
        f"{NEW_PAGE_TAG}\n"
        "CERTIFICATS D'ASSURANCE RESPONABILITÉ AUTOMOBILE\n"
        "CONSERVER AVEC VOTRE PERMIS DE CONDUIRE\n"
        f"Pour télécharger vos certiﬁcats dans l'application Wallet (disponible seulement sur iPhone) :\n"
        f"\t1. Connectez-vous à Espace client à partir de votre iPhone, au {FAKE_INSURER_WEBSITE}/espaceclient.\n"
        f"\t2. Dans la section Auto, habitation et véhicules de loisirs, cliquez sur « Obtenir "
        f"un certiﬁcat d'assurance », puis sur « Ajouter à Apple Wallet ».\n"
        f"Vous devrez mettre à jour vos certiﬁcats à chaque renouvellement ou modiﬁcation de votre police "
        f"d'assurance en les téléchargeant de nouveau dans l'application Wallet.\n"
        f"{FAKE_INSURER_NAME} ne peut pas garantir l'acceptation des certiﬁcats en format électronique par "
        f"tous les corps policiers. Assurez-vous de toujours en garder une version papier avec vous.\n\n"
    )

    if self.protections.is_protected("FAQ27Coverage") or self.protections.is_protected("FAQ27aCoverage"):
        text += (
            f"CERTIFICAT D'ASSURANCE POUR LOCATION DE VOITURE\n"
            f"(Valide au Canada et aux États-Unis d'Amérique seulement)\n"
            f"Application particulière de l'avenant F.A.Q. 27 : Responsabilité civile du fait de dommages causés à "
            f"des véhicules dont l'assuré désigné n'est pas propriétaire (incluant les véhicules fournis par un "
            f"employeur).\n"
            f"En cas de vol ou d'endommagement d'un véhicule de location, {FAKE_INSURER_NAME} s'engage "
            f"à rembourser, à l'Assuré ou au propriétaire du véhicule, le montant des dommages réellement encourus "
            f"et dont l'Assuré ou son conjoint s'est rendu responsable en vertu d'un contrat de location, le tout "
            f"jusqu'à concurrence de : 75 000 $\n\n"
            f"CAR RENTAL INSURANCE CERTIFICATE\n"
            f"(Valid only in Canada and the United States of America)\n"
            f"Speciﬁc application of Q.E.F. endorsement B 27 : Civil liability resulting from damage caused to "
            f"vehicles of which named insured is not owner (including vehicles provided by an employer).\n"
            f"In the event of theft or damage to a rental vehicle, {FAKE_INSURER_NAME} "
            f"agrees to refund the Insured or the owner of the vehicle, the amount of damage actually "
            f"incurred and for which the Insured or his/her spouse assumes liability under a rental "
            f"contract, in all up to: $75,000\n"
        )

    # Two certificates
    text += back_of_certificates_text
    text += back_of_certificates_text
    text += NEW_PAGE_TAG + "\n"

    return text



from typing import Dict

from sdv.single_table import TVAESynthesizer

from ..model.synthetic_protections_model import SyntheticProtectionsModel


class TVAESyntheticProtectionsModelWrapper(SyntheticProtectionsModel):
    def __init__(self, model_path: str) -> None:
        """
        A Synthetic protection model using a TVAE architecture.
        See CTGAN (https://pypi.org/project/ctgan/) for more details.
        """
        self.model = TVAESynthesizer.load(model_path)

    def sample(self, num_sample: int, **kwargs) -> Dict:
        """
        Sample num_sample of a synthetic protections using a pre-trained model.

        num_sample (int): Number of sample to generate.
        kwargs (dict): Other parameters to use for sampling.

        Return:
            A dictionary of the synthetic protections.
        """
        # We set output_file_path to "disable" since it generate a tempfile and for a large number of single file
        # tempfile generation, it seems to raise an error. "Disabling" the tempfile creation avoid this error for an
        # unknown reason.
        return self.model.sample(num_rows=num_sample, output_file_path="disable", **kwargs).to_dict(orient="records")[0]



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq32_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 32\n"
        "Véhicules à but uniquement récréatif\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat d'assurance.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N : \n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Cet avenant modifie le contrat d'assurance pour le véhicule automobile à but uniquement récréatif qui :\n"
        "\t est expressément désigné à l'article 3 de la section « Conditions particulières » du contrat "
        "d'assurance; ou\n"
        "\t fait partie des « véhicules assurés » du contrat d'assurance.\n"
        "Véhicule automobile à but uniquement récréatif\n"
        "Un « véhicule automobile à but uniquement récréatif » vise, entre autres, tout véhicule "
        "automobile qui est ou non de fabrication commerciale, et qui est assimilable :\n"
        "\t aux « dune buggies »;\n"
        "\t aux mini-motos;\n"
        "\t aux mini-voitures;\n"
        "\t aux motoneiges; et\n"
        "aux véhicules automobiles tout-terrain.\n"
        "Description des modifications\n"
        "1. Chapitre A : le paragraphe E. de l'article 2. intitulé « Véhicules assurés » est remplacé "
        "par le paragraphe suivant :\n"
        "« E. Sauf si elle est désignée à la section « Conditions particulières », toute remorque "
        "(qu'elle appartienne ou non à l'assuré désigné) utilisée avec un véhicule automobile "
        "à but uniquement récréatif qui est :\n"
        "\t du même type que celui désigné à la section « Conditions particulières »; et\n"
        "\t couvert par le contrat d'assurance. »\n"
        "2. Conditions générales : le paragraphe a) de l'article 7. intitulé « Usages interdits "
        "d'un véhicule assuré » est remplacé par le paragraphe suivant :\n"
        "« a) Elles ne sont pas autorisées à conduire par la loi; »\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "3. Les expressions « véhicule automobile » et « véhicule automobile utilisé à des "
        "fins personnelles » sont remplacées partout dans le contrat d'assurance par l'expression suivante :\n"
        "« véhicule automobile à but uniquement récréatif du même type que celui désigné à la section "
        "« Conditions particulières » ».\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from risc_generator import FPQ1Contract, Insuree
from risc_generator.content import fr, en
from tests.domain.contract.contract_base import ContractTestBase

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractDeclarationTest(ContractTestBase):
    def test_givenAFRContractInsureeNoSuspensionOrClaims_whenGenerateItem6Text_thenReturnProperFRText(self):
        a_suspension = False
        a_number_of_claims_past_years = 0
        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=self.a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_6_text(
            insuree_name=fpq1_contract.insuree.name,
            insuree_birth_date=fpq1_contract.insuree.date_of_birth,
            has_suspension=fpq1_contract.insuree.suspension,
            number_of_claims_past_years=fpq1_contract.insuree.number_of_claims_past_years,
        )

        a_insuree_name = "A Name"
        a_insuree_birth_date = "2020-01-01"

        expected = (
            "Article 6. DÉCLARATIONS IMPORTANTES POUR L'ANALYSE DU RISQUE\n"
            f"\t Conducteur principal : {a_insuree_name} Date de naissance : {a_insuree_birth_date}\n"
            "\t Aucune révocation ou suspension de permis n'ont été déclarées au cours des trois dernières années\n"
            "\t Aucun sinistre, aucune réclamation n'a été déclaré au cours des cinq dernières années\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractInsureeWithASuspensionButNoClaims_whenGenerateItem6Text_thenReturnProperFRText(self):
        a_suspension = True
        a_number_of_claims_past_years = 0
        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=self.a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_6_text(
            insuree_name=fpq1_contract.insuree.name,
            insuree_birth_date=fpq1_contract.insuree.date_of_birth,
            has_suspension=fpq1_contract.insuree.suspension,
            number_of_claims_past_years=fpq1_contract.insuree.number_of_claims_past_years,
        )

        a_insuree_name = "A Name"
        a_insuree_birth_date = "2020-01-01"

        expected = (
            "Article 6. DÉCLARATIONS IMPORTANTES POUR L'ANALYSE DU RISQUE\n"
            f"\t Conducteur principal : {a_insuree_name} Date de naissance : {a_insuree_birth_date}\n"
            "\t Au moins une révocation ou suspension de permis a été déclarée au cours des trois dernières années\n"
            "\t Aucun sinistre, aucune réclamation n'a été déclaré au cours des cinq dernières années\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractInsureeWithoutASuspensionButWithClaims_whenGenerateItem6Text_thenReturnProperFRText(self):
        a_suspension = False
        a_number_of_claims_past_years = 2
        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=self.a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_6_text(
            insuree_name=fpq1_contract.insuree.name,
            insuree_birth_date=fpq1_contract.insuree.date_of_birth,
            has_suspension=fpq1_contract.insuree.suspension,
            number_of_claims_past_years=fpq1_contract.insuree.number_of_claims_past_years,
        )

        a_insuree_name = "A Name"
        a_insuree_birth_date = "2020-01-01"

        expected = (
            "Article 6. DÉCLARATIONS IMPORTANTES POUR L'ANALYSE DU RISQUE\n"
            f"\t Conducteur principal : {a_insuree_name} Date de naissance : {a_insuree_birth_date}\n"
            "\t Aucune révocation ou suspension de permis n'ont été déclarées au cours des trois dernières années\n"
            "\t 2 sinistre(s) ou réclamation(s) ont été déclarés au cours des cinq dernières années\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractInsureeWithASuspensionAndClaims_whenGenerateItem6Text_thenReturnProperFRText(self):
        a_suspension = True
        a_number_of_claims_past_years = 2
        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=self.a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_6_text(
            insuree_name=fpq1_contract.insuree.name,
            insuree_birth_date=fpq1_contract.insuree.date_of_birth,
            has_suspension=fpq1_contract.insuree.suspension,
            number_of_claims_past_years=fpq1_contract.insuree.number_of_claims_past_years,
        )

        a_insuree_name = "A Name"
        a_insuree_birth_date = "2020-01-01"

        expected = (
            "Article 6. DÉCLARATIONS IMPORTANTES POUR L'ANALYSE DU RISQUE\n"
            f"\t Conducteur principal : {a_insuree_name} Date de naissance : {a_insuree_birth_date}\n"
            "\t Au moins une révocation ou suspension de permis a été déclarée au cours des trois dernières années\n"
            "\t 2 sinistre(s) ou réclamation(s) ont été déclarés au cours des cinq dernières années\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractInsureeNoSuspensionOrClaims_whenGenerateItem6Text_thenReturnProperENText(self):
        a_suspension = False
        a_number_of_claims_past_years = 0
        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=self.a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_6_text(
            insuree_name=fpq1_contract.insuree.name,
            insuree_birth_date=fpq1_contract.insuree.date_of_birth,
            has_suspension=fpq1_contract.insuree.suspension,
            number_of_claims_past_years=fpq1_contract.insuree.number_of_claims_past_years,
        )

        a_insuree_name = "A Name"
        a_insuree_birth_date = "2020-01-01"

        expected = (
            "Item 6. IMPORTANT STATEMENTS FOR ANALYZING THE RISK\n"
            f"\t Main driver: {a_insuree_name} Date of birth: {a_insuree_birth_date}\n"
            "\t No revocation or suspension of licence has been reported over the past three years\n"
            "\t No loss or claim has been reported over the past five years\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractInsureeWithASuspensionButNoClaims_whenGenerateItem6Text_thenReturnProperENText(self):
        a_suspension = True
        a_number_of_claims_past_years = 0
        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=self.a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_6_text(
            insuree_name=fpq1_contract.insuree.name,
            insuree_birth_date=fpq1_contract.insuree.date_of_birth,
            has_suspension=fpq1_contract.insuree.suspension,
            number_of_claims_past_years=fpq1_contract.insuree.number_of_claims_past_years,
        )

        a_insuree_name = "A Name"
        a_insuree_birth_date = "2020-01-01"

        expected = (
            "Item 6. IMPORTANT STATEMENTS FOR ANALYZING THE RISK\n"
            f"\t Main driver: {a_insuree_name} Date of birth: {a_insuree_birth_date}\n"
            "\t At least one revocation or suspension of licence has been reported over the past three years\n"
            "\t No loss or claim has been reported over the past five years\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractInsureeWithoutASuspensionButWithClaims_whenGenerateItem6Text_thenReturnProperENText(self):
        a_suspension = False
        a_number_of_claims_past_years = 2
        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=self.a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_6_text(
            insuree_name=fpq1_contract.insuree.name,
            insuree_birth_date=fpq1_contract.insuree.date_of_birth,
            has_suspension=fpq1_contract.insuree.suspension,
            number_of_claims_past_years=fpq1_contract.insuree.number_of_claims_past_years,
        )

        a_insuree_name = "A Name"
        a_insuree_birth_date = "2020-01-01"

        expected = (
            "Item 6. IMPORTANT STATEMENTS FOR ANALYZING THE RISK\n"
            f"\t Main driver: {a_insuree_name} Date of birth: {a_insuree_birth_date}\n"
            "\t No revocation or suspension of licence has been reported over the past three years\n"
            "\t 2 loss or claim has been reported over the past five years\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractInsureeWithASuspensionAndClaims_whenGenerateItem6Text_thenReturnProperENText(self):
        a_suspension = True
        a_number_of_claims_past_years = 2
        a_insuree = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=a_suspension,
            number_of_claims_past_years=a_number_of_claims_past_years,
            association_rebate=self.a_association_rebate,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_6_text(
            insuree_name=fpq1_contract.insuree.name,
            insuree_birth_date=fpq1_contract.insuree.date_of_birth,
            has_suspension=fpq1_contract.insuree.suspension,
            number_of_claims_past_years=fpq1_contract.insuree.number_of_claims_past_years,
        )

        a_insuree_name = "A Name"
        a_insuree_birth_date = "2020-01-01"

        expected = (
            "Item 6. IMPORTANT STATEMENTS FOR ANALYZING THE RISK\n"
            f"\t Main driver: {a_insuree_name} Date of birth: {a_insuree_birth_date}\n"
            "\t At least one revocation or suspension of licence has been reported over the past three years\n"
            "\t 2 loss or claim has been reported over the past five years\n"
        )

        self.assertEqual(expected, actual)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq5a_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 5a\n"
        "Vehicles leased or under a contact of leasing Changes\n"
        "when owner and one lessee are mentioned as insureds\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement changes the insurance contract:\n"
        "\t when the specified vehicle leased or under a contract of leasing; and\n"
        "\t when the owner and one lessee of the vehicle are mentioned as insureds under the insurance contract.\n"
        "The expression ''named insured'' will then be replaced by ''named lessee'' in the definition of "
        "the following\n"
        "''insured vehicles'':\n"
        "\t Vehicle of which the named insured has recently become the owner.\n"
        "\t Temporary substitute vehicle.\n"
        "\t Vehicle of which the named insured is not the owner.\n"
        "\t Trailer or semi-trailer of which the named insured is the owner.\n"
        "\t Trailer or semi-trailer of which the named insured is not the owner and that is used "
        "in connection with a vehicle insured under the insurance contract.\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from unittest import TestCase

from risc_generator import endorsement_coverage_parsing

INCLUDE = "include"
EXCLUDE = "exclude"


class EndorsementCoverageParsingTests(TestCase):
    def test_givenACoverageWithEndorsement_whenEndorsementCoverageParsing_thenReturnProperIncludeExcludeProtections(
        self,
    ):
        synthetic_coverage = {
            "LiabilityCoverage": 1,
            "PropertyDamageB1Coverage": 0,
            "PropertyDamageB2Coverage": 0,
            "PropertyDamageB3Coverage": 0,
            "PropertyDamageB4Coverage": 0,
            "FAQ5ACoverage": 1,
            "FAQ13ACoverage": 1,
        }
        actual = endorsement_coverage_parsing(synthetic_coverage)

        expected = {
            "FAQ5ACoverage": INCLUDE,
            "FAQ13ACoverage": INCLUDE,
        }

        self.assertDictEqual(actual, expected)

    def test_givenACoverageWithoutEndorsement_whenEndorsementCoverageParsing_thenReturnProperIncludeExcludeProtections(
        self,
    ):
        synthetic_coverage = {
            "LiabilityCoverage": 1,
            "PropertyDamageB1Coverage": 0,
            "PropertyDamageB2Coverage": 0,
            "PropertyDamageB3Coverage": 0,
            "PropertyDamageB4Coverage": 0,
        }
        actual = endorsement_coverage_parsing(synthetic_coverage)

        expected = {}

        self.assertDictEqual(actual, expected)

    def test_givenACoverageWithEndorsementExclude_whenEndorsementCoverageParsing_thenReturnProperExcludeProtections(
        self,
    ):
        synthetic_coverage = {
            "LiabilityCoverage": 1,
            "PropertyDamageB1Coverage": 0,
            "PropertyDamageB2Coverage": 0,
            "PropertyDamageB3Coverage": 0,
            "PropertyDamageB4Coverage": 0,
            "FAQ5ACoverage": 0,
            "FAQ13ACoverage": 0,
        }
        actual = endorsement_coverage_parsing(synthetic_coverage)

        expected = {
            "FAQ5ACoverage": EXCLUDE,
            "FAQ13ACoverage": EXCLUDE,
        }

        self.assertDictEqual(actual, expected)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq5a_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 5a\n"
        "Véhicules loués ou pris en crédit-bail\n"
        "Modifications\n"
        "lorsque le propriétaire et un locataire ou crédit-preneur\n"
        "sont désignés comme assurés\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat d'assurance.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Cet avenant modifie le contrat d'assurance :\n"
        "\t lorsque le véhicule visé est loué ou pris en crédit-bail; et\n"
        "\t lorsque le propriétaire et un locataire ou crédit-preneur de ce véhicule sont désignés comme "
        "assurés au contrat d'assurance.\n"
        "L'expression « assuré désigné » est alors remplacée par « locataire désigné » ou, selon le cas, "
        "par « crédit-preneur désigné », dans la définition des « véhicules assurés » suivants :\n"
        "\t Véhicule dont l'assuré désigné est nouvellement propriétaire.\n"
        "\t Véhicule de remplacement temporaire.\n"
        "\t Véhicule dont l'assuré désigné n'est pas propriétaire.\n"
        "\t Remorque ou semi-remorque dont l'assuré désigné est propriétaire.\n"
        "\t Remorque ou semi-remorque dont l'assuré désigné n'est pas propriétaire et qui est "
        "utilisée avec un véhicule assuré au contrat d'assurance.\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq31_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 31\n"
        "Équipement n'appartenant pas à l'assuré désigné\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à "
        "cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N : \n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Cet avenant étend les garanties du contrat d'assurance à l'équipement suivant, dont "
        "l'assuré désigné n'est pas propriétaire :\n"
        "Conditions d'application\n"
        "L'équipement doit être normalement fixé au véhicule visé. De plus, l'assuré désigné doit "
        "avoir un pouvoir de direction ou de gestion sur cet équipement, ou en avoir la garde.\n"
        "1. Application au chapitre A\n"
        "La garantie du chapitre A est étendue aux conséquences financières que peut "
        "subir la personne assurée lorsqu'elle est civilement responsable d'un dommage causé à une "
        "autre personne du fait de l'équipement décrit ci-dessus.\n"
        "2. Application au chapitre B\n"
        "La garantie du chapitre B applicable au véhicule visé est étendue :\n"
        "\t aux dommages directs et accidentels causés à l'équipement décrit ci-dessus;\n"
        "\t à la disparition de cet équipement.\n"
        "Pour le chapitre B, l'indemnité est payable jusqu'à concurrence de la « valeur au jour du "
        "sinistre » de l'équipement et pour un montant maximum de $. Elle est payable "
        "conjointement à l'assuré désigné et à  selon leurs intérêts respectifs.\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



import datetime
from unittest.mock import patch

from risc_generator import FPQ1Contract, Protections
from tests.domain.contract.contract_base import ContractTestBase

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractHeaderTest(ContractTestBase):
    def setUp(self) -> None:
        self.a_protections = Protections(protections=self.a_dict_of_protections)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAFRContractPreAuthorize_whenGenerateHeaderText_thenReturnProperFRText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_header_text()

        a_date = "Le 2 janvier 2022\n\n\n"
        a_client_number = "Numéro client: 1234567\n"
        a_vehicle = "AMaker - 2020\n"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            f"{a_date}"
            "ÉMISSION\n"
            f"{a_client_number}"
            "Police / Durée du terme:\n"
            "001 / 12 mois\n\n"
            "JAMAIS SEUL POUR VOS ASSURANCES\n\n"
            "Vous trouverez vos documents d'assurance aux pages suivantes.\n"
            "Nous vous suggérons de les lire attentivement afin de vous assurer qu'ils sont conformes.\n"
            "Si une page Instructions importantes fait partie de ces documents, veuillez y porter une "
            "attention particulière, puisqu'elle comporte des renseignements essentiels au sujet de "
            "vos protections.\n"
            "N'hésitez pas à communiquer avec nous pour toute question ou commentaire.\n\n"
            "NOUS JOINDRE\n"
            "Service à la clientèle\n"
            "123 456-7890\n"
            "1 800 123-4567\n"
            "Télécopieur: 012 345-6789\n"
            "service.client@assureur.ca\n"
            "8 h à 20 h du lundi au vendredi\n"
            "8 h 30 à 16 h le samedi\n"
            "Réclamations\n"
            "098 765-4321\n"
            "1 800 765-4321\n"
            "Urgence 7 jours/24 heures\n"
            "Assistance juridique\n"
            "1 800 012-3456\n"
            "8 h 30 à 19 h du lundi au jeudi\n"
            "8 h 30 à 17 h le vendredi\n"
            "Votre succursale\n"
            "221 B Baker Street\n"
            "Ville (Province)\nA1A 1A1\n"
            "8 h 30 à 17 h du lundi au vendredi\n"
            "assureur.ca\n"
            "Vos documents d'assurance\n"
            "(Table des matières)\n"
            "Instructions importantes\n"
            "Assurance automobile\n"
            "Numéro police: 001\n"
            f"{a_vehicle}"
            f"Certificats d'assurance\n"
            f"(Documents à conserver)\n"
            f"Avantages Client\n"
            f"(Description de vos bénéfices client)\n"
            f"Libellés et avenants\n"
            f"{a_new_page}\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAFRContractBill_whenGenerateHeaderText_thenReturnProperFRText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method="facture",  # FR bill payment method
            protections=self.a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_header_text()

        a_date = "Le 2 janvier 2022\n\n\n"
        a_client_number = "Numéro client: 1234567\n"
        a_vehicle = "AMaker - 2020\n"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            f"{a_date}"
            "ÉMISSION\n"
            f"{a_client_number}"
            "Police / Durée du terme:\n"
            "001 / 12 mois\n\n"
            "JAMAIS SEUL POUR VOS ASSURANCES\n\n"
            "Vous trouverez vos documents d'assurance aux pages suivantes.\n"
            "Nous vous suggérons de les lire attentivement afin de vous assurer qu'ils sont conformes.\n"
            "Si une page Instructions importantes fait partie de ces documents, veuillez y porter une "
            "attention particulière, puisqu'elle comporte des renseignements essentiels au sujet de "
            "vos protections.\n"
            "N'hésitez pas à communiquer avec nous pour toute question ou commentaire.\n\n"
            "NOUS JOINDRE\n"
            "Service à la clientèle\n"
            "123 456-7890\n"
            "1 800 123-4567\n"
            "Télécopieur: 012 345-6789\n"
            "service.client@assureur.ca\n"
            "8 h à 20 h du lundi au vendredi\n"
            "8 h 30 à 16 h le samedi\n"
            "Réclamations\n"
            "098 765-4321\n"
            "1 800 765-4321\n"
            "Urgence 7 jours/24 heures\n"
            "Assistance juridique\n"
            "1 800 012-3456\n"
            "8 h 30 à 19 h du lundi au jeudi\n"
            "8 h 30 à 17 h le vendredi\n"
            "Votre succursale\n"
            "221 B Baker Street\n"
            "Ville (Province)\nA1A 1A1\n"
            "8 h 30 à 17 h du lundi au vendredi\n"
            "assureur.ca\n"
            "Vos documents d'assurance\n"
            "(Table des matières)\n"
            "Instructions importantes\n"
            "Assurance automobile\n"
            "Numéro police: 001\n"
            f"{a_vehicle}"
            f"Certificats d'assurance\n"
            f"(Documents à conserver)\n"
            f"Facturation\n"
            f"Avantages Client\n"
            f"(Description de vos bénéfices client)\n"
            f"Libellés et avenants\n"
            f"{a_new_page}\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAENContractPreAuthorize_whenGenerateHeaderText_thenReturnProperENText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_header_text()

        a_date = "January 2, 2022\n\n\n"
        a_client_number = "Client N: 1234567\n"
        a_vehicle = "AMaker - 2020\n"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            f"{a_date}"
            "ISSUE\n"
            f"{a_client_number}"
            "Policy / Term:\n"
            "001 / 12 month\n\n"
            "ALWAYS BY YOUR SIDE FOR YOUR INSURANCE\n\n"
            "Please find enclosed your insurance documents.\n"
            "We recommend that you read them carefully to ensure that they are correct.\n"
            "If an Important Instructions page is included with these documents, please read through it carefully "
            "because it contains crucial information regarding your coverage.\n"
            "Feel free to contact us if you have any questions or comments.\n\n"
            "CONTACT US\n"
            "Customer Service\n"
            "123 456-7890\n"
            "1 800 123-4567\n"
            "Fax: 012 345-6789\n"
            "service.client@assureur.ca\n"
            f"8 a.m. to 8 p.m.\n"
            f"Monday to Friday\n"
            f"8:30 a.m. to 4 p.m., Saturday\n"
            "Claims\n"
            "098 765-4321\n"
            "1 800 765-4321\n"
            "24/7 emergency\n"
            "Legal Assistance\n"
            "1 800 012-3456\n"
            f"8:30 a.m. to 7 p.m.\n"
            f"Monday to Thursday\n"
            f"8:30 a.m. to 5 p.m., Friday\n"
            "Your branch\n"
            "221 B Baker Street\n"
            "Ville (Province)\nA1A 1A1\n"
            f"8:30 a.m. to 5 p.m.\n"
            f"Monday to Friday\n"
            "assureur.ca\n"
            "Your Insurance Documents\n"
            "(Table of Contents)\n"
            "Important Instructions\n"
            "Automobile Insurance\n"
            "Policy number: 001\n"
            f"{a_vehicle}"
            f"Insurance Certificates\n"
            f"(Documents to be kept on file)\n"
            f"Client Advantages\n"
            f"(Description of your client benefits)\n"
            f"Wording and Endorsements\n"
            f"{a_new_page}\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAENContractBill_whenGenerateHeaderText_thenReturnProperENText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_bill_payment_method,
            protections=self.a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_header_text()

        a_date = "January 2, 2022\n\n\n"
        a_client_number = "Client N: 1234567\n"
        a_vehicle = "AMaker - 2020\n"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            f"{a_date}"
            "ISSUE\n"
            f"{a_client_number}"
            "Policy / Term:\n"
            "001 / 12 month\n\n"
            "ALWAYS BY YOUR SIDE FOR YOUR INSURANCE\n\n"
            "Please find enclosed your insurance documents.\n"
            "We recommend that you read them carefully to ensure that they are correct.\n"
            "If an Important Instructions page is included with these documents, please read through it carefully "
            "because it contains crucial information regarding your coverage.\n"
            "Feel free to contact us if you have any questions or comments.\n\n"
            "CONTACT US\n"
            "Customer Service\n"
            "123 456-7890\n"
            "1 800 123-4567\n"
            "Fax: 012 345-6789\n"
            "service.client@assureur.ca\n"
            f"8 a.m. to 8 p.m.\n"
            f"Monday to Friday\n"
            f"8:30 a.m. to 4 p.m., Saturday\n"
            "Claims\n"
            "098 765-4321\n"
            "1 800 765-4321\n"
            "24/7 emergency\n"
            "Legal Assistance\n"
            "1 800 012-3456\n"
            f"8:30 a.m. to 7 p.m.\n"
            f"Monday to Thursday\n"
            f"8:30 a.m. to 5 p.m., Friday\n"
            "Your branch\n"
            "221 B Baker Street\n"
            "Ville (Province)\nA1A 1A1\n"
            f"8:30 a.m. to 5 p.m.\n"
            f"Monday to Friday\n"
            "assureur.ca\n"
            "Your Insurance Documents\n"
            "(Table of Contents)\n"
            "Important Instructions\n"
            "Automobile Insurance\n"
            "Policy number: 001\n"
            f"{a_vehicle}"
            f"Insurance Certificates\n"
            f"(Documents to be kept on file)\n"
            f"Invoice\n"
            f"Client Advantages\n"
            f"(Description of your client benefits)\n"
            f"Wording and Endorsements\n"
            f"{a_new_page}\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq41_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 41\n"
        "Change to deductibles\n"
        "(Section B)\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N: \n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement makes the following changes to the deductible amounts under Section "
        "B, as entered in Item 4, ''Declarations'' of the insurance contract:\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text
